/*!*****************************************************
*
*    ex11.c
*    ======
*
*    EXgtpt();      Interface routine for GETPRT
*    EXgtyp();      Interface routine for GETTYP
*    EXgint();      Interface routine for GETINT
*    EXgflt();      Interface routine for GETFLT
*    EXgstr();      Interface routine for GETSTR
*    EXgvec();      Interface routine for GETVEC
*    EXgref();      Interface routine for GETREF
*    EXtime();      Interface routine for TIME
*    EXuphd();      Interface routine for UPDHDR
*    EXinpt();      Interface routine for INPMT
*    EXinfn();      Interface routine for INFNAME
*    EXdirl();      Returns "dir"-list
*
*    This file is part of the VARKON Execute  Library.
*    URL:  http://www.varkon.com
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1998, Johan Kjellander, johan@microform.se
*
*********************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#ifdef V3_X11
#include "../../WP/include/WP.h"
#endif
#include "../include/EX.h"
#include <string.h>
#include <time.h>

#ifdef WIN32
#include <io.h>
#endif 


extern GMDATA v3dbuf;
extern short ialy,ialx;

#ifdef WIN32
extern int msilse();
#endif

/*!******************************************************/

        short EXgtpt(
        DBId    *id,
        GMPRT   *part)

/*      Interface-rutin fr GETPRT. Hmtar part-data
 *      ur GM.
 *
 *      In: id     => Pekare till partens identitet.
 *          part   => Pekare till GMPRT.
 *
 *      Ut: *part  => Part-post.
 *
 *      FV:  0     => Ok.
 *          EX1402 => Hittar ej storheten
 *          EX1412 => Otillten typ fr denna operation
 *
 *      (C)microform ab 2/2/86 J. Kjellander
 *
 ******************************************************!*/

  {
    DBptr  la;
    DBetype  typ;

/*
***verstt ID -> la.
*/
    if ( DBget_pointer('I',id,&la,&typ) < 0 )
       return(erpush("EX1402",""));
    if ( typ != PRTTYP ) return(erpush("EX1412",""));
/*
***Ls part-posten och returnera part-data.
*/
    DBread_part(part,la);

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXgtyp(
        DBId    *id,
        DBshort  pnum,
        PMLITVA *typ)

/*      Interface-rutin fr GETTYP. Hmtar typ-data
 *      ur GM.
 *
 *      In: id     => Pekare till partens ID
 *          pnum   => Parameterns orningsnummer.
 *          typ    => Pekare till short.
 *
 *      Ut: *typ   => Typ av parameter.
 *
 *      FV:  0     => Ok.
 *          EX1612 => Parametern finns ej
 *
 *      (C)microform ab 1/4/86 J. Kjellander
 *
 ******************************************************!*/

  {
    short    status;
    GMPRT    part;
    GMPDAT   data;
    PMPATLOG typarr[V2MPMX];

/*
***Ls part-posten.
*/
    if ( (status=EXgtpt(id,&part)) < 0 ) return(status);
/*
***Ls data-posten.
*/
    DBread_part_parameters(&data,typarr,NULL,part.dtp_pt,part.dts_pt);
/*
***Kolla om parametern pnum finns.
*/
    if ( pnum > data.npar_pd || pnum < 1 ) return(erpush("EX1612",""));
/*
***Returnera typ.
*/
    typ->lit.int_va = typarr[pnum-1].log_id;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXgint(
        DBId    *id,
        DBshort  pnum,
        PMLITVA *intval)

/*      Interface-rutin fr GETINT. Hmtar heltalsparam.
 *      ur GM.
 *
 *      In: id      => Pekare till partens ID
 *          pnum    => Parameterns ordningsnummer.
 *          intval  => Pekare till heltal.
 *
 *      Ut: *intval  => Heltalet.
 *
 *      FV:  0      => Ok.
 *          EX1612 => Parametern finns ej
 *          EX1622 => Parametern har inte typen %s
 *
 *      (C)microform ab 1/4/86 J. Kjellander
 *
 *      23/10/86 v3dbuf, J. Kjellander
 *
 ******************************************************!*/

  {
    short    status;
    GMPRT    part;
    GMPDAT   data;
    PMPATLOG typarr[V2MPMX];
    v2int   *valpek;

/*
***Ls part-posten.
*/
    if ( (status=EXgtpt(id,&part)) < 0 ) return(status);
/*
***Ls data-posten.
*/
    DBread_part_parameters(&data,typarr,v3dbuf.valarr,part.dtp_pt,part.dts_pt);
/*
***Kolla om parametern pnum finns och har rtt typ.
*/
    if ( pnum > data.npar_pd || pnum < 1 ) return(erpush("EX1612",""));
    if ( typarr[pnum-1].log_id != C_INT_VA ) return(erpush("EX1622","<int>"));
/*
***Returnera vrde.
*/
    valpek = (v2int *) &v3dbuf.valarr[typarr[pnum-1].log_offs];
    intval->lit.int_va = *valpek;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXgflt(
        DBId    *id,
        DBshort  pnum,
        PMLITVA *fltval)

/*      Interface-rutin fr GETFLT. Hmtar flyttals-param.
 *      ur GM.
 *
 *      In: id      => Pekare till partens ID
 *          pnum    => Parameterns ordningsnummer
 *          fltval  => Pekare till flyttal
 *
 *      Ut: *fltval = Flyttal
 *
 *      FV:      0 => Ok.
 *          EX1612 => Parametern finns ej
 *          EX1622 => Parametern har inte typen %s
 *
 *      (C)microform ab 2/4/86 J. Kjellander
 *
 *      23/10/86 v3dbuf, J. Kjellander
 *
 ******************************************************!*/

  {
    short    status;
    GMPRT    part;
    GMPDAT   data;
    PMPATLOG typarr[V2MPMX];
    v2float *valpek;

/*
***Ls part-posten.
*/
    if ( (status=EXgtpt(id,&part)) < 0 ) return(status);
/*
***Ls data-posten.
*/
    DBread_part_parameters(&data,typarr,v3dbuf.valarr,part.dtp_pt,part.dts_pt);
/*
***Kolla om parametern pnum finns och har rtt typ.
*/
    if ( pnum > data.npar_pd || pnum < 1 ) return(erpush("EX1612",""));
    if ( typarr[pnum-1].log_id != C_FLO_VA )
                  return(erpush("EX1622","<float>"));
/*
***Returnera vrde.
*/
    valpek = (v2float *) &v3dbuf.valarr[typarr[pnum-1].log_offs];
    fltval->lit.float_va = *valpek;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXgstr(
        DBId    *id,
        DBshort  pnum,
        PMLITVA *strval)

/*      Interface-rutin fr GETSTR. Hmtar strng-param
 *      ur GM.
 *
 *      In: id     => Pekare till partens ID
 *          pnum   => Parameterns ordningsnummer
 *          strval => Pekare till strng
 *
 *      Ut: *strval  => Strng
 *
 *      FV:      0 => Ok.
 *          EX1612 => Parametern finns ej
 *          EX1622 => Parametern har inte typen %s
 *
 *      (C)microform ab 2/4/86 J. Kjellander
 *
 *      23/10/86 v3dbuf, J. Kjellander
 *
 ******************************************************!*/

  {
    short    status;
    GMPRT    part;
    GMPDAT   data;
    PMPATLOG typarr[V2MPMX];

/*
***Ls part-posten.
*/
    if ( (status=EXgtpt(id,&part)) < 0 ) return(status);
/*
***Ls data-posten.
*/
    DBread_part_parameters(&data,typarr,v3dbuf.valarr,part.dtp_pt,part.dts_pt);
/*
***Kolla om parametern pnum finns och har rtt typ.
*/
    if ( pnum > data.npar_pd || pnum < 1 ) return(erpush("EX1612",""));
    if ( typarr[pnum-1].log_id != C_STR_VA )
                          return(erpush("EX1622","<string>"));
/*
***Returnera vrde.
*/
    strcpy(strval->lit.str_va,&v3dbuf.valarr[typarr[pnum-1].log_offs]);

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXgvec(
        DBId    *id,
        DBshort  pnum,
        PMLITVA *vecval)

/*      Interface-rutin fr GETVEC. Hmtar vector-param.
 *      ur GM.
 *
 *      In: id     => Pekare till partens ID
 *          pnum   => Parameterns ordningsnummer
 *          vecval => Pekare till v2vecva
 *
 *      Ut: *vecval => Vektor
 *
 *      FV:      0 => Ok.
 *          EX1612 => Parametern finns ej
 *          EX1622 => Parametern har inte typen %s
 *
 *      (C)microform ab 2/4/86 J. Kjellander
 *
 *      23/10/86 v3dbuf, J. Kjellander
 *
 ******************************************************!*/

  {
    short    status;
    GMPRT    part;
    GMPDAT   data;
    PMPATLOG typarr[V2MPMX];
    V2VECVA *vecp;

/*
***Ls part-posten.
*/
    if ( (status=EXgtpt(id,&part)) < 0 ) return(status);
/*
***Ls data-posten.
*/
    DBread_part_parameters(&data,typarr,v3dbuf.valarr,part.dtp_pt,part.dts_pt);
/*
***Kolla om parametern pnum finns och har rtt typ.
*/
    if ( pnum > data.npar_pd || pnum < 1 ) return(erpush("EX1612",""));
    if ( typarr[pnum-1].log_id != C_VEC_VA ) 
                         return(erpush("EX1622","<vector>"));
/*
***Returnera vrde.
*/
    vecp = (V2VECVA *) &v3dbuf.valarr[typarr[pnum-1].log_offs];
    vecval->lit.vec_va.x_val = vecp->x_val;
    vecval->lit.vec_va.y_val = vecp->y_val;
    vecval->lit.vec_va.z_val = vecp->z_val;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXgref(
        DBId    *id,
        DBshort  pnum,
        PMLITVA *refval)

/*      Interface-rutin fr GETVEC. Hmtar ref-param.
 *      ur GM.
 *
 *      In: id     => Pekare till partens ID
 *          pnum   => Parameterns ordningsnummer
 *          vecval => Pekare till vrde
 *
 *      Ut: *refval => Referens
 *
 *      FV:      0 => Ok.
 *          EX1612 => Parametern finns ej
 *          EX1622 => Parametern har inte typen %s
 *
 *      (C)microform ab 1/6/86 J. Kjellander
 *
 *      23/10/86 v3dbuf, J. Kjellander
 *
 ******************************************************!*/

  {
    short    status,i;
    GMPRT    part;
    GMPDAT   data;
    PMPATLOG typarr[V2MPMX];
    V2REFVA *refp;

/*
***Ls part-posten.
*/
    if ( (status=EXgtpt(id,&part)) < 0 ) return(status);
/*
***Ls data-posten.
*/
    DBread_part_parameters(&data,typarr,v3dbuf.valarr,part.dtp_pt,part.dts_pt);
/*
***Kolla om parametern pnum finns och har rtt typ.
*/
    if ( pnum > data.npar_pd || pnum < 1 ) return(erpush("EX1612",""));
    if ( typarr[pnum-1].log_id != C_REF_VA ) 
                         return(erpush("EX1622","<ref>"));
/*
***Returnera vrde.
*/
    refp = (V2REFVA *) &v3dbuf.valarr[typarr[pnum-1].log_offs];

    for ( i=0;; ++i )
      {
      refval->lit.ref_va[i].seq_val = (refp+i)->seq_val;
      refval->lit.ref_va[i].ord_val = (refp+i)->ord_val;
      if ( (refp+i)->p_nextre == NULL )
        {
        refval->lit.ref_va[i].p_nextre = NULL;
        break;
        }
      else
        {
        refval->lit.ref_va[i].p_nextre = &refval->lit.ref_va[i+1];
        }
      }

    return(0);
  }

/********************************************************/
/*!******************************************************/


        short EXtime(
        DBshort *y,
        DBshort *mo,
        DBshort *d,
        DBshort *h,
        DBshort *mi,
        DBshort *s)

/*      Interface-rutin fr TIME. Hmtar datum
 *      och tid.
 *
 *      In: Inget.
 *
 *      Ut: *y  => r.
 *          *mo => Mnad.
 *          *d  => Dag i mnaden.
 *          *h  => Timme.
 *          *mi => Minut.
 *          *s  => Sekund.
 *
 *      FV:  0     => Ok.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {
    long       reltim;
    struct tm *timpek;
/*
***Ls av klockan.
*/
    reltim = time ((long *)0);
    timpek = localtime(&reltim);

    *y  = (short)timpek->tm_year;
    *mo = (short)(timpek->tm_mon+1);
    *d  = (short)timpek->tm_mday;
    *h  = (short)timpek->tm_hour;
    *mi = (short)timpek->tm_min;
    *s  = (short)timpek->tm_sec;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXuphd(
        DBId    *id,
        DBshort blank,
        DBshort level,
        DBshort pen)

/*      Interface-rutin fr UPDHDR. Skriver header-data
 *      till angiven storhet.
 *
 *      In: id       => Storhetens identitet.
 *          blank    => Tnd/Slckt.
 *          level    => Niv.
 *          pen      => Pennummer.
 *
 *      Ut: Inget.
 *
 *      FV:  0     => Ok.
 *          EX1402 => Storheten finns ej.
 *
 *      (C)microform ab 17/10/88 J. Kjellander
 *
 *      15/3/92  refcnt, J. Kjellander
 *
 ******************************************************!*/

  {
    DBptr  la;
    DBetype  typ;
    GMRECH header;

/*
***verstt ID -> la.
*/
    if ( DBget_pointer('I',id,&la,&typ) < 0 )
               return(erpush("EX1402",""));
/*
***Ls gamla headern.
*/
    DBread_header(&header,la);
/*
***Lite indatakontroll.
*/
    if ( blank != 0 ) header.blank  = 1; else header.blank = 0;
    if ( pen    < 0 ) header.pen    = 0; else header.pen = pen;

    if ( level  < 0  ||  level > NT1SIZ-1 )
      header.level  = 0;
    else
      header.level = level;
/*
***Uppdatera.
*/
    return(DBupdate_header(&header,la));
  }

/********************************************************/
/*!******************************************************/

        short EXinpt(
        char   *pmt,
        char   *dstr,
        DBshort ntkn,
        char   *istr)

/*      Interface-rutin fr INPMT. Skriver en frga med
 *      ev. defaultvrde p inmatningsraden och lser in 
 *      en strng med ev. maxlngd som svar.
 *
 *      In: pmt  = Promtstrng.
 *          dstr = Defaultstrng.
 *          ntkn = Fltvidd.
 *
 *      Ut: *istr = Inlst strng.
 *
 *      FV:  0     => Ok.
 *
 *      (C)microform ab 10/11/88 J. Kjellander
 *
 ******************************************************!*/

  {
    short fwdth;

    if ( ntkn == 0 ) fwdth = V3STRLEN;
    else fwdth = ntkn;

    igmvac(ialx,ialy);
    igglin(pmt,dstr,&fwdth,istr);

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXinfn(
        char *pmt,
        char *dstr,
        char *path,
        char *pattern,
        char *name)

/*      Interface-rutin fr INFNAME.
 *
 *      In: pmt     = Rubriktext.
 *          dstr    = Default i editboxen.
 *          path    = Filkatalog.
 *          pattern = Skstrng.
 *          name    = Pekare till utdata.
 *
 *      Ut: *name = Filnamn.
 *
 *      FV:  0     => Ok.
 *
 *      (C)microform ab 1988-04-01 J. Kjellander
 *
 ******************************************************!*/

  {
   short status;
   char *pekarr[1000],strarr[20000];
   DBint nfiles;

/*
***Skapa filfrteckning.
*/
   EXdirl(path,pattern,1000,20000,pekarr,strarr,&nfiles);
/*
***Dialogbox.
*/
#ifdef V3_X11
   status = wpilse(20,20,pmt,dstr,pekarr,-1,nfiles,name);
#endif
#ifdef WIN32
   status = (short)msilse(20,20,pmt,dstr,pekarr,-1,nfiles,name);
#endif
/*
***Hur gick det ?
*/
   if ( status < 0 ) *name = '\0';

   return(0);
  }

/********************************************************/
/*!******************************************************/

        short EXdirl(
        char  *inpath,
        char  *pattern,
        DBint  maxant,
        DBint  maxsiz,
        char  *pekarr[],
        char  *strarr,
        DBint *nf)

/*      Gr "ls" p en filkatalog. Denna rutin hette 
 *      tidigare igdir().
 *
 *      In:
 *          inpath  = Skvgbeskrivning, tex. "/usr/v3/pid".
 *                    Ev. slash p slutet gr bra.
 *          pattern = Skstrng, tex. *.RES eller * eller A*B.
 *          maxant  = Max antal filer (pekarr:s storlek)
 *          maxsiz  = Max antal tecken (buf:s storlek).
 *          strarr  = Plats att lagra filnamn.
 *
 *      Ut:
 *          pekarr = Array med nf stycken pekare till filnamn.
 *          nf     = Antal filer.
 *
 *      FV:  0 = Ok.
 *
 *      (C)microform ab 2/11/95 J. Kjellander
 *
 *      1998-04-10 typ->pattern, J.Kjellander
 *
 ******************************************************!*/

  {
   char  lscmd[V3PTHLEN+1],path[V3PTHLEN+1];
   char *actpek;
   int   actsiz,i;


#ifdef WIN32
   struct _finddata_t fildata;
   long               fp;

/*
***Initiering.
*/
   actpek = strarr;
   actsiz = 0;
  *nf     = 0;
/*
***Fr skerhets skull kopierar vi path till en
***lokal buffert och strippar ev. '\' och/eller '.' sist.
***path = "." eller ".\" kan tex. frekomma.
*/
   strcpy(path,inpath);

   i = strlen(path) - 1;
   if ( path[i] == '\\' ) path[i] = '\0';

   i = strlen(path) - 1;
   if ( path[i] == '.' ) path[i] = '\0';
/*
***Skapa hela skkommandot.
*/
   if ( strlen(path) > 0 )
     {
     strcpy(lscmd,path);
     strcat(lscmd,"\\");
     strcat(lscmd,pattern);
     }
   else
     {
     strcpy(lscmd,pattern);
     }
/*
***ppna directoryt och hmta 1:a filnamnet.
*/
   if ( (fp=_findfirst(lscmd,&fildata)) == -1L ) return(0);
/*
***Sen tar vi resten ocks.
*/
   else do
     {
     if ( strcmp(fildata.name,".")   != 0  &&
          strcmp(fildata.name,".." ) != 0 )
       {
       strcpy(actpek,fildata.name);
       pekarr[*nf] = actpek;
       actpek += strlen(fildata.name)+1;
       actsiz += strlen(fildata.name)+1;
      *nf += 1;
       }
     } while ( *nf < maxant  &&  actsiz < (maxsiz-JNLGTH-5)  &&
                                _findnext(fp,&fildata) == 0 );

   _findclose(fp);

return(0);
#endif







#ifdef UNIX
   extern FILE *popen();
   extern int   pclose();

   char  buf[V3PTHLEN+1];
   int   n;
   FILE *fp;

/*
***Initiering.
*/
   actpek = strarr;
   actsiz = 0;
  *nf     = 0;
/*
***Fr skerhets skull kopierar vi path till en
***lokal buffert och strippar ev. '/' och/eller '.' sist.
***path = "." eller "./" kan tex. frekomma.
*/
   strcpy(path,inpath);

   i = strlen(path) - 1;
   if ( path[i] == '/' ) path[i] = '\0';

   i = strlen(path) - 1;
   if ( path[i] == '.' ) path[i] = '\0';
/*
***En pipe till ls.
*/
   if ( strlen(path) > 0 )
     {
     strcpy(lscmd,"cd ");
     strcat(lscmd,path);
     strcat(lscmd,";ls ");
     strcat(lscmd,pattern);
     }
   else
     {
     strcpy(lscmd,"ls ");
     strcat(lscmd,pattern);
     }

   if ( (fp=popen(lscmd,"r")) == NULL ) return(erpush("IG0442",lscmd));
/*
***Ls en rad i taget. Max 1000 projekt och max 20000 tecken totalt.
*/
   while ( fgets(buf,V3PTHLEN,fp) != NULL  && 
                     *nf < maxant  &&  actsiz+JNLGTH+5 < maxsiz )
     {
     if ( (n=strlen(buf)) > 0 ) buf[n-1] = '\0';
     strcpy(actpek,buf);
     pekarr[*nf] = actpek;
     actpek += strlen(buf)+1;
     actsiz += strlen(buf)+1;
    *nf += 1;
     }

   pclose(fp);

   return(0);
#endif
  }

/********************************************************/
