/**********************************************************************
*
*    eval13.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evopen();     Evaluerar OPEN
*    evclos();     Evaluerar CLOSE
*    evoutl();     Evaluerar OUTLIN
*    v3isop();     r fil ppen ?
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../GP/include/GP.h"

extern FILE *popen();
extern int   pclose();

typedef struct open_file
{
char       name[V3STRLEN+1];
V2FILE    *fptr;
}OPEN_FILE;

#define MAXOFILES 50

static OPEN_FILE filtab[MAXOFILES] =
     {{"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL},
      {"",NULL},{"",NULL},{"",NULL},{"",NULL},{"",NULL}};

/* filtab har plats fr MAXOFILES samtidigt ppna filer. */

static int fndfree();
static int close_all();
static int fndfil(char *name);
static int fndptr(V2FILE *fptr);

/*!******************************************************/

        short evopen(PMPARVA pv[])

/*      Evaluerar OPEN.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till funktionsvrde.
 *
 *      Felkod: IN3253 = Felaktig mode = %s
 *              IN5322 = Filen %s r redan ppen
 *              IN5332 = Max antal filer redan ppna
 *
 *      (C)microform ab 1997-01-30 J. Kjellander
 *
 ******************************************************!*/

 {
   int        i;
   char       pmode[2];
   char      *name,*mode;
   V2FILE    *f;

/*
***Filens namn ev med path.
*/
   name = pv[3].par_va.lit.str_va;
/*
***Kolla att den inte redan r ppnad.
*/
   if ( v3isop(name) ) return(erpush("IN5322",name));
/*
***Kolla att det finns plats i filtab.
*/
   if ( (i=fndfree()) == -1 ) return(erpush("IN5332",name));
/*
***Filvariabel.
*/
   f = pv[1].par_va.lit.fil_va;
/*
***Kolla mode.
*/
   mode = pv[2].par_va.lit.str_va;

   if ( sticmp("w",mode) == 0 )
     {
     f->mode[0] = 'w';
     f->mode[1] = '\0';
     }
   else if ( sticmp("wp",mode) == 0 )
     {
     f->mode[0] = 'w';
     f->mode[1] = 'p';
     f->mode[2] = '\0';
     }
   else if ( sticmp("r",mode) == 0 )
     {
     f->mode[0] = 'r';
     f->mode[1] = '\0';
     }
   else if ( sticmp("rp",mode) == 0 )
     {
     f->mode[0] = 'r';
     f->mode[1] = 'p';
     f->mode[2] = '\0';
     }
   else if ( sticmp("u",mode) == 0 )
     {
     f->mode[0] = 'u';
     f->mode[1] = '\0';
     }
   else if ( sticmp("a",mode) == 0 )
     {
     f->mode[0] = 'a';
     f->mode[1] = '\0';
     }
   else return(erpush("IN3253",mode));
/*
***ppna fr update.
*/
   if ( f->mode[0] == 'u' )
     {
     if ( (f->fp=fopen(name,"r+")) == NULL )
       {
       if ( (f->fp=fopen(name,"w+")) == NULL ) f->iostat = -2;
       else                                    f->iostat =  0;
       }
     else f->iostat = 0;
     }
/*
***Pipe open.
*/
   else if ( f->mode[1] == 'p' )
     {
     if ( f->mode[0] == 'r' ) strcpy(pmode,"r");
     else                     strcpy(pmode,"w");
#ifdef WIN32
     if ( (f->fp = _popen(name,pmode) ) == NULL ) f->iostat = -2;
     else                                         f->iostat =  0;
#else
     if ( (f->fp =  popen(name,pmode) ) == NULL ) f->iostat = -2;
     else                                         f->iostat =  0;
#endif
     }
/*
***Vanlig read/write/append mode.
*/
   else
     {
     if ( (f->fp=fopen(name,f->mode)) == NULL ) f->iostat = -2;
     else                                       f->iostat =  0;
     }
/*
***Hur gick det ?
*/
   if ( f->iostat == 0 )
     {
     strcpy(filtab[i].name,name);
     filtab[i].fptr = f;
     f->open = TRUE;
     }
   else f->open = FALSE;

   return(0);
 }

/********************************************************/
/*!******************************************************/

        short evclos(
        PMPARVA  pv[],
        short    np)

/*      Evaluerar CLOSE.
 *
 *      In: pv   => Pekare till array med parametervrdet.
 *          np   => Antal parametrar, 0 eller 1.
 *
 *      Felkod: IN5342 = Kan ej stnga, filen ej ppen
 *              IN5353 = Systemfel, kan ej stnga %s
 *
 *      (C)microform ab 1997-01-30 J. Kjellander
 *
 ******************************************************!*/

 {
   int     i,status;
   V2FILE *f;

/*
***On ingen parameter skickats med skall alla filer stngas.
*/
   if ( np == 0 ) return((short)close_all());
/*
***Annars finns det en parameter = Filpekare.
*/
   f = pv[1].par_va.lit.fil_va;
/*
***Kolla att den r ppnad.
*/
   if ( (i=fndptr(f)) == -1 ) return(erpush("IN5342",""));
/*
***Stng den.
*/
   if ( f->open )
     {
/*
***Pipe.
*/
     if ( f->mode[1] == 'p' )
       {
#ifdef WIN32
       if ( _pclose(f->fp) < 0 ) f->iostat = -3;
       else                      f->iostat =  0;
#else
       if (  pclose(f->fp) < 0 ) f->iostat = -3;
       else                      f->iostat =  0;
#endif
       }
/*
***Vanlig fil.
*/
     else
       {
       if ( fclose(f->fp) < 0 ) f->iostat = -3;
       else                     f->iostat =  0;
       }
     }
/*
***f->open = FALSE. Skall ju inte kunna hnda numera men fr
***vara kvar nd.
*/
   else f->iostat = -10;
/*
***Stnga en fil br man alltid kunna gra om den var ppen.
*/
   if ( f->iostat < 0 ) status = erpush("IN5353",filtab[i].name);
   else                 status = 0;
/*
***Oavsett hur det gick noterar vi stngt i filvariabel och filtabell.
*/
   f->open = FALSE;

  *filtab[i].name = '\0';
   filtab[i].fptr = NULL;

   return(status);
 }

/********************************************************/
/*!******************************************************/

        short evoutl( PMPARVA pv[])

/*      Evaluerar OUTLIN.
 *
 *      In: pv   => Pekare till array med parametervrdet.
 *
 *      Felkod: IN5362 = Filen ej ppen
 *
 *      (C)microform ab 1997-01-30 J. Kjellander
 *
 ******************************************************!*/

 {
   int     i;
   V2FILE *f;

/*
***Filpekare.
*/
   f = pv[1].par_va.lit.fil_va;
/*
***Kolla att den r ppnad.
*/
   if ( (i=fndptr(f)) == -1 ) return(erpush("IN5362",""));
/*
***Kolla att den r ppnad med rtt mode.
*/
   if ( !( ( (f->mode[0] == 'w') ||
             (f->mode[0] == 'a') ||
             (f->mode[0] == 'u') )  && f->open ) )
     {
     f->iostat = -4;
     return(0);
     }
/*
***Special fr standard output.
*/
   if ( f->fp == stdout ) gpansi();
/*
***Skriv.
*/
   fputc('\n',f->fp);
/*
***Special fr standard output.
*/
   if ( f->fp == stdout ) igflsh();
/*
***Slut.
*/
   f->iostat = 0;

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        bool v3isop(char *path)

/*      Kollar om fil r ppen.
 *
 *      In: path = Filnamn ev. med path.
 *
 *      FV: TRUE  = Filen r ppen.
 *          FALSE = Filen r inte ppen.
 *
 *      (C)microform ab 1997-02-05 J. Kjellander
 *
 ******************************************************!*/

 {
   if ( fndfil(path) == -1 ) return(FALSE);
   else                      return(TRUE);
 }

/********************************************************/
/*!******************************************************/

 static int close_all()

/*      Stnger all ppna filer i filtab.
 *
 *      FV:   0.
 *
 *      (C)microform ab 1998-01-01 J. Kjellander
 *
 *      1998-02-19 Bugfix, J.Kjellander
 *
 ******************************************************!*/

 {
   int     i;
   V2FILE *f;

/*
***G igenom filtab och sk upp alla ppna filer.
*/
   for ( i=0; i<MAXOFILES; ++i )
     {
     if ( *filtab[i].name != '\0' )
       {
       f = filtab[i].fptr;
/*
***En ppen fil !
*/
       if ( f->open )
         {
/*
***Pipe.
*/
         if ( f->mode[1] == 'p' )
           {
#ifdef WIN32
           if ( _pclose(f->fp) < 0 ) f->iostat = -3;
#else
           if (  pclose(f->fp) < 0 ) f->iostat = -3;
#endif
           else                      f->iostat =  0;
           }
/*
***Vanlig fil.
*/
         else
           {
           if ( fclose(f->fp) < 0 ) f->iostat = -3;
           else                     f->iostat =  0;
           }
         f->open = FALSE;
        *filtab[i].name = '\0';
         filtab[i].fptr = NULL;
         }
       }
     }

   return(0);
 }

/********************************************************/
/*!******************************************************/

 static int fndfil(char *name)

/*      Sker upp en fil i filtab.
 *
 *      In: name = Filens namn ev. med path
 *
 *      FV:   -1 = Finns inte
 *          >= 0 = Index i filtab.
 *
 *      (C)microform ab 1997-01-30 J. Kjellander
 *
 ******************************************************!*/

 {
   int i;

#ifdef WIN32
   for ( i=0; i<MAXOFILES; ++i )
     if ( *filtab[i].name != '\0'  &&
          sticmp(name,filtab[i].name) == 0 ) return(i);
#else
   for ( i=0; i<MAXOFILES; ++i )
     if ( *filtab[i].name != '\0'  &&
          strcmp(name,filtab[i].name) == 0 ) return(i);
#endif

   return(-1);
 }

/********************************************************/
/*!******************************************************/

 static int fndptr(V2FILE *fptr)

/*      Sker upp en fil i filtab.
 *
 *      In: name = Filens fptr.
 *
 *      FV:   -1 = Finns inte
 *          >= 0 = Index i filtab.
 *
 *      (C)microform ab 1997-01-30 J. Kjellander
 *
 ******************************************************!*/

 {
   int i;

   for ( i=0; i<MAXOFILES; ++i )
     if ( fptr == filtab[i].fptr ) return(i);

   return(-1);
 }

/********************************************************/
/*!******************************************************/

 static int fndfree()

/*      Sker upp ledig plats i filtab.
 *
 *      FV:   -1 = filtab full.
 *          >= 0 = Index till ledig plats i filtab.
 *
 *      (C)microform ab 1997-01-30 J. Kjellander
 *
 ******************************************************!*/

 {
   int i;

   for ( i=0; i<MAXOFILES; ++i )
     if ( *filtab[i].name == '\0' ) return(i);

   return(-1);
 }

/********************************************************/
