/**********************************************************************
*
*    eval4.c
*    =======
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evldgm();     Evaluerar LOAD_GM
*    evsvgm();     Evaluerar SAVE_GM
*    evldjb();     Evaluerar LOAD_JOB
*    evsvjb();     Evaluerar SAVE_JOB
*    evsvgm();     Evaluerar SAVE_GM
*    evbllv();     Evaluerar BLANK_LEV
*    evublv();     Evaluerar UNBLANK_LEV
*    evgtlv();     Evaluerar GET_LEV
*    evgtvi();     Evaluerar GET_VIEW
*    evcrvi();     Evaluerar CRE_VIEW
*    evacvi();     Evaluerar ACT_VIEW
*    evscvi();     Evaluerar SCL_VIEW
*    evcevi();     Evaluerar CEN_VIEW
*    evhdvi();     Evaluerar HIDE_VIEW
*    evprvi();     Evaluerar PERP_VIEW
*    evervi();     Evaluerar ERASE_VIEW
*    evrpvi();     Evaluerar REP_VIEW
*    evcavi();     Evaluerar CACC_VIEW
*    evplvi();     Evaluerar PLOT_VIEW
*    evplwi();     Evaluerar PLOT_WIN
*    evmsiz();     Evaluerar MSIZE_VIEW
*    evgrid();     Evaluerar GRID_VIEW
*    evgrix();     Evaluerar GRIDX_VIEW
*    evgriy();     Evaluerar GRIDY_VIEW
*    evgrdx();     Evaluerar GRIDDX_VIEW
*    evgrdy();     Evaluerar GRIDDY_VIEW
*    evtime();     Evaluerar TIME
*    evlsin();     Evaluerar LST_INI
*    evlsex();     Evaluerar LST_EXI
*    evlsln();     Evaluerar LST_LIN
*    evpspt();     Evaluerar PSH_PMT
*    evpopt();     Evaluerar POP_PMT
*    evcrts();     Evaluerar CRE_TSTR
*    evcrmy();     Evaluerar CRE_MEN
*    evpsmy();     Evaluerar PSH_MEN
*    evpomy();     Evaluerar POP_MEN
*    evgtmy();     Evaluerar GET_MEN
*    evlmdf();     Evaluerar LOAD_MDF
*    evexst();     Evaluerar EXESTAT
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#ifdef V3_X11
#include "../../WP/include/WP.h"
#endif
#include "../../EX/include/EX.h"

/*!******************************************************/

        short evldgm(PMPARVA *pv)

/*      Evaluerar proceduren LOAD_GM.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/6/86 J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXldgm(&pv[1].par_va.lit.str_va[0]));
  }

/********************************************************/
/*!******************************************************/

        short evsvgm(PMPARVA *pv)

/*      Evaluerar proceduren SAVE_GM.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/6/86 J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXsvgm(&pv[1].par_va.lit.str_va[0]));
  }

/********************************************************/
/*!******************************************************/

        short evldjb(PMPARVA *pv, short npar)

/*      Evaluerar proceduren LOAD_JOB.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 14/10/88 R. Svedin
 *
 ******************************************************!*/

  {
    short  status,i;

    for ( i=1; i<npar; ++i )
      {
      if ( (status=EXldjb(pv[1].par_va.lit.str_va,
                     (short)pv[i+1].par_va.lit.int_va)) < 0 ) return(status);
      }
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evsvjb(PMPARVA *pv)

/*      Evaluerar proceduren SAVE_JOB.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 14/10/88 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXsvjb(&pv[1].par_va.lit.str_va[0]));
  }

/********************************************************/
/*!******************************************************/

        short evbllv(PMPARVA *pv)

/*      Evaluerar proceduren BLANK_LEV.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/6/86 J. Kjellander
 *
 *      17/1-95  Multifnster, J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXbllv(pv[1].par_va.lit.int_va,
                  pv[2].par_va.lit.int_va));
  }

/********************************************************/
/*!******************************************************/

        short evublv(PMPARVA *pv)

/*      Evaluerar proceduren UNBLANK_LEV.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/6/86 J. Kjellander
 *
 *      17/1-95  Multifnster, J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXublv(pv[1].par_va.lit.int_va,
                  pv[2].par_va.lit.int_va));
  }

/********************************************************/
/*!******************************************************/

        short evgtlv(PMPARVA *pv)

/*      Evaluerar proceduren GET_LEV.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/6/86 J. Kjellander
 *
 ******************************************************!*/

  {
    bool    blank;
    char    name[NIVNLN+1];
    short   status;
    PMLITVA litval[2];

/*
***Hmta niv-info.
*/
    if ( (status=EXgtlv((short)pv[1].par_va.lit.int_va,&blank,name)) < 0 )
          return(status);;
/*
***Kopiera till PMLITVA.
*/
    if ( blank ) litval[0].lit.int_va = 1;
    else         litval[0].lit.int_va = 0;
    strcpy(litval[1].lit.str_va,name);
/*
***Skriv parametervrden till motsvarande MBS-variabler.
*/
    evwval(litval,2,pv);
/*
***Slut.
*/
    return(0);

  }

/********************************************************/
/*!******************************************************/

        short evgtvi(PMPARVA *pv)

/*      Evaluerar proceduren GET_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden. 
 *
 *      Ut: Inget.
 *
 *      Felkod:
 *
 *      (C)microform ab 20/1-95 J. Kjellander
 *
 ******************************************************!*/

  {
   short   status,i,j,index[2];
   char    vynamn[V3STRLEN+1];
   double  skala,xmin,ymin,xmax,ymax,persp;
   v2int   win_id;
   DBTmat  vymat;
   PMLITVA litval[16];

   win_id = pv[1].par_va.lit.int_va;

#ifdef V3_X11
   wpgtvi(win_id,vynamn,&skala,&xmin,&ymin,&xmax,&ymax,&vymat,&persp);
#else
#ifdef WIN32
   msgtvi(win_id,vynamn,&skala,&xmin,&ymin,&xmax,&ymax,&vymat,&persp);
#else
   return(-2);
#endif
#endif
/*
***Vynamn,skala,modellfnster.
*/
   strcpy(litval[0].lit.str_va,vynamn);
   litval[1].lit.float_va = skala;
   litval[2].lit.float_va = xmin;
   litval[3].lit.float_va = ymin;
   litval[4].lit.float_va = xmax;
   litval[5].lit.float_va = ymax;

   evwval(litval,6,pv);
/*
***Transformationsmatris.
*/
   litval[0].lit.float_va  = vymat.g11;
   litval[1].lit.float_va  = vymat.g12;
   litval[2].lit.float_va  = vymat.g13;
   litval[3].lit.float_va  = vymat.g14;

   litval[4].lit.float_va  = vymat.g21;
   litval[5].lit.float_va  = vymat.g22;
   litval[6].lit.float_va  = vymat.g23;
   litval[7].lit.float_va  = vymat.g24;

   litval[8].lit.float_va  = vymat.g31;
   litval[9].lit.float_va  = vymat.g32;
   litval[10].lit.float_va = vymat.g33;
   litval[11].lit.float_va = vymat.g34;

   litval[12].lit.float_va = vymat.g41;
   litval[13].lit.float_va = vymat.g42;
   litval[14].lit.float_va = vymat.g43;
   litval[15].lit.float_va = vymat.g44;

   for ( i=0; i<4; ++i )
     {
     index[0] = i+1;
     for ( j=0; j<4; ++j )
       {
       index[1] = j+1;
       status = inwvar(pv[8].par_ty, pv[8].par_va.lit.adr_va,
                                            2, index, &litval[4*i+j]);
       }
     }
/*
***Perspektiv.
*/
   litval[0].lit.float_va = persp;
   inwvar(pv[9].par_ty,pv[9].par_va.lit.adr_va,0,NULL,&litval[0]);
/*
***Slut.
*/
   return(0);
  }

/********************************************************/
/*!******************************************************/

        short evcrvi(PMPARVA *pv)

/*      Evaluerar proceduren CRE_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      Felkoder: IN5062 = Fel typ av indata.
 *
 *      (C)microform ab 30/6/86 R. Svedin
 *
 *      10/10/86 Tagit bort param. fr vyfnster, R. Svedin
 *      29/9/92  Matris-vyer, J. Kjellander
 *
 ******************************************************!*/

  {
    VYVEC    bpos;

/*
***Betraktelseposition (VECTOR) eller koordinatsystem (REF).
*/
    switch ( pv[2].par_va.lit_type )
      {
      case C_VEC_VA:
/*
***Betraktelseposition.
*/
      bpos.x_vy = pv[2].par_va.lit.vec_va.x_val;
      bpos.y_vy = pv[2].par_va.lit.vec_va.y_val;
      bpos.z_vy = pv[2].par_va.lit.vec_va.z_val;
      return(EXcrvp(pv[1].par_va.lit.str_va,&bpos));
      break;
/*
***Koordinatsystem.
*/
      case C_REF_VA:
      return(EXcrvc(pv[1].par_va.lit.str_va,&pv[2].par_va.lit.ref_va[0]));
/*
***Felaktig typ av indata.
*/
      default:
      return(erpush("IN5062",""));
      }

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evacvi(PMPARVA *pv)

/*      Evaluerar proceduren ACT_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/8/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXacvi(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.int_va));
  }

/********************************************************/
/*!******************************************************/

        short evscvi(PMPARVA *pv)

/*      Evaluerar proceduren SCL_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/8/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXscvi(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.float_va));
  }

/********************************************************/
/*!******************************************************/

        short evcevi(PMPARVA *pv)

/*      Evaluerar proceduren CEN_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 4/8/86 R. Svedin
 *
 ******************************************************!*/

   {
    return(EXcevi(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.vec_va.x_val,
                  pv[2].par_va.lit.vec_va.y_val));
  }

/********************************************************/
/*!******************************************************/

        short evhdvi(PMPARVA *pv, short npar)

/*      Evaluerar proceduren HIDE_VIEW.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          npar => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV:      0 = OK.
 *          AVBRYT = <CTRL>c i gphide().
 *
 *      Felkoder: IN2262 = Filnamn saknas.
 *                IN2272 = Felaktig kod.
 *                IN2282 = Kan ej skapa fil %s.
 *
 *      (C)microform ab 21/3/89 J. Kjellander
 *
 *      29/5/91 Placering, J. Kjellander
 *
 ******************************************************!*/

   {
    short  code,status;
    DBVector *origo;
    FILE  *fp;

/*
***Skrm, fil eller bda ?
*/
    code = (short)pv[2].par_va.lit.int_va;
/*
***Nollpunkt. Om ingen getts, ta skrmens nedre vnstra hrn.
*/
    if ( npar == 4 ) origo = (DBVector * )&pv[4].par_va.lit.vec_va;
    else origo = NULL;

    if ( code == 1 )
      return(EXhdvi(pv[1].par_va.lit.str_va,TRUE,FALSE,NULL,NULL));
    else if ( npar < 3 )
      return(erpush("IN2262",""));
    else
      {
      if ( (fp=fopen(pv[3].par_va.lit.str_va,"w+")) == NULL )
        return(erpush("IN2282",pv[3].par_va.lit.str_va));
      else if ( code == 2 )
        status = EXhdvi(pv[1].par_va.lit.str_va,FALSE,TRUE,fp,origo);
      else if ( code == 3 )
        status = EXhdvi(pv[1].par_va.lit.str_va,TRUE,TRUE,fp,origo);
      else
        status = erpush("IN2272","");
      fclose(fp);
      }

    return(status);
  }

/********************************************************/
/*!******************************************************/

        short evprvi(PMPARVA *pv)

/*      Evaluerar proceduren PERP_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 21/3/89 J. Kjellander
 *
 ******************************************************!*/

   {
    return(EXprvi(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.float_va));
  }

/********************************************************/
/*!******************************************************/

        short evervi(PMPARVA *pv)

/*      Evaluerar proceduren ERASE_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 30/6/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXervi(pv[1].par_va.lit.int_va));
  }

/********************************************************/
/*!******************************************************/

        short evrpvi(PMPARVA *pv, short npar)

/*      Evaluerar proceduren REP_VIEW.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          npar => Antal parametrar 2, 1 eller 0.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 30/6/86 R. Svedin
 *
 *      19/10/93 Autozoom, J. Kjellander
 *      27/12/94 Multifnster, J. Kjellander
 *
 ******************************************************!*/

  {
    v2int azoom,win_id;

    if ( npar == 0 )
      {
      azoom = 0;
      win_id = GWIN_MAIN;
      }
    else if ( npar == 1 )
      {
      azoom = pv[1].par_va.lit.int_va;
      win_id = GWIN_MAIN;
      }
    else
      {
      azoom = pv[1].par_va.lit.int_va;
      win_id = pv[2].par_va.lit.int_va;
      }

    return(EXrpvi(azoom,win_id));
  }

/********************************************************/
/*!******************************************************/

        short evcavi(PMPARVA *pv)

/*      Evaluerar proceduren CACC_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 4/8/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXcavi(pv[1].par_va.lit.float_va));
  }

/********************************************************/
/*!******************************************************/

        short evplvi(PMPARVA *pv)

/*      Evaluerar proceduren PLOT_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Inget.
 *
 *      (C)microform ab 3/11/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXplvi(&pv[1].par_va.lit.str_va[0],
                  &pv[2].par_va.lit.str_va[0]));
  }

/********************************************************/
/*!******************************************************/

        short evplwi(PMPARVA *pv, short np)

/*      Evaluerar proceduren PLOT_WIN.
 *
 *      In: pv => Pekare till array med parametervrden
 *          np => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Inget.
 *
 *      (C)microform ab 16/2/88 J. Kjellander
 *
 ******************************************************!*/

  {

/*
***Om antalet parametrar r 4 har plot-origo utryckligen
***getts som indata. ( 4:e parametern )
*/
    if ( np == 4 )
      {
      return(EXplwi((DBVector *)&pv[1].par_va.lit.vec_va,
                    (DBVector *)&pv[2].par_va.lit.vec_va,
                              pv[3].par_va.lit.str_va,
                    (DBVector *)&pv[4].par_va.lit.vec_va));
      }
/*
***Om inte skall nedre vnstra hrnet p plotomrdet
***anvndas. ( 1:a parametern )
*/
    else
      {
      return(EXplwi((DBVector *)&pv[1].par_va.lit.vec_va,
                    (DBVector *)&pv[2].par_va.lit.vec_va,
                              pv[3].par_va.lit.str_va,
                    (DBVector *)&pv[1].par_va.lit.vec_va));
      }

  }

/********************************************************/
/*!******************************************************/

        short evmsiz(PMPARVA *pv)

/*      Evaluerar proceduren MSIZE_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Inget.
 *
 *      (C)microform ab 8/8/93
 *
 ******************************************************!*/

  {
   short   status;
   VY      modvy;
   PMLITVA litval;

/*
***Hmta modellens storlek.
*/
    if ( (status=igmsiz(&modvy)) < 0 ) return(status);
/*
***Om modell saknas eller saknar utstrckning returnerar vi nollor.
*/
    if ( (modvy.vywin[2] < modvy.vywin[0]) &&
         (modvy.vywin[3] < modvy.vywin[1]) )
      {
      modvy.vywin[0] = modvy.vywin[1] =
      modvy.vywin[2] = modvy.vywin[3] = 0.0;
      }
/*
***Skriv in resultatet i p1 och p2.
*/    
    litval.lit.vec_va.x_val = modvy.vywin[0];
    litval.lit.vec_va.y_val = modvy.vywin[1];
    inwvar(pv[1].par_ty,pv[1].par_va.lit.adr_va,0,NULL,&litval);
    litval.lit.vec_va.x_val = modvy.vywin[2];
    litval.lit.vec_va.y_val = modvy.vywin[3];
    inwvar(pv[2].par_ty,pv[2].par_va.lit.adr_va,0,NULL,&litval);

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evgrid(PMPARVA *pv)

/*      Evaluerar proceduren GRID_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      (C)microform ab 1996-02-12
 *
 ******************************************************!*/

  {
    extern bool  rstron;
    extern gmflt rstrox,rstroy,rstrdx,rstrdy;
    extern short gpdrrs(),gpdlrs();

/*
***Rastret skall tndas. Om det redan var tnt gr vi ingenting.
***Fr att ndra ett raster skall man allts frst slcka det,
***sedan ndra det och sist tnda det igen.
*/
    if ( pv[1].par_va.lit.int_va == 1 )
      {
      if ( !rstron )
        {
        rstron = TRUE;
        gpdrrs(rstrox,rstroy,rstrdx,rstrdy);
        }
      }
/*
***Rastret skall slckas. Om det redan var slckt gr vi heller
***ingenting.
*/
    else if ( pv[1].par_va.lit.int_va == 0 )
      {
      if ( rstron )
        {
        rstron = FALSE;
        gpdlrs(rstrox,rstroy,rstrdx,rstrdy);
        }
      }
/*
***Om parametern varken var 1 (Tnd) eller 0 (Slck) gr vi ingenting.
*/

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evgrix(PMPARVA *pv)

/*      Evaluerar proceduren GRIDX_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      (C)microform ab 1996-02-12
 *
 ******************************************************!*/

  {
    extern gmflt rstrox;

    rstrox = pv[1].par_va.lit.float_va;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evgriy(PMPARVA *pv)

/*      Evaluerar proceduren GRIDY_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      (C)microform ab 1996-02-12
 *
 ******************************************************!*/

  {
    extern gmflt rstroy;

    rstroy = pv[1].par_va.lit.float_va;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evgrdx(PMPARVA *pv)

/*      Evaluerar proceduren GRIDDX_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      (C)microform ab 1996-02-12
 *
 ******************************************************!*/

  {
    extern gmflt rstrdx;

    rstrdx = pv[1].par_va.lit.float_va;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evgrdy(PMPARVA *pv)

/*      Evaluerar proceduren GRIDDY_VIEW.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      (C)microform ab 1996-02-12
 *
 ******************************************************!*/

  {
    extern gmflt rstrdy;

    rstrdy = pv[1].par_va.lit.float_va;

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evtime(PMPARVA *pv)

/*      Evaluerar proceduren TIME.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {
    short   status,i;
    short   y,mo,d,h,mi,s;
    PMLITVA litval[6];

/*
***Hmta datum och tid.
*/
    if ( (status=EXtime(&y,&mo,&d,&h,&mi,&s)) < 0 )
      return(status);
/*
***Kopiera parametervrden till PMLITVA.
*/
    litval[0].lit.int_va = y;
    litval[1].lit.int_va = mo;
    litval[2].lit.int_va = d;
    litval[3].lit.int_va = h;
    litval[4].lit.int_va = mi;
    litval[5].lit.int_va = s;

/*
***Skriv parametervrden till motsvarande MBS variabler.
*/
    for ( i=0; i<6; ++i )
      {
      inwvar(pv[i+1].par_ty, pv[i+1].par_va.lit.adr_va,
             0, NULL, &litval[i]);
      }

    return(0);

  }

/********************************************************/
/*!******************************************************/

        short evlsin(PMPARVA *pv)

/*      Evaluerar proceduren LST_INI.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXlsin(&pv[1].par_va.lit.str_va[0]));
  }

/********************************************************/
/*!******************************************************/

        short evlsex(PMPARVA *pv)

/*      Evaluerar proceduren LST_EXI.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXlsex());
  }

/********************************************************/
/*!******************************************************/

        short evlsln(PMPARVA *pv)

/*      Evaluerar proceduren LST_LIN.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXlsln(pv[1].par_va.lit.str_va));
  }

/********************************************************/
/*!******************************************************/

        short evpspt(PMPARVA *pv)

/*      Evaluerar proceduren PSH_PMT.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXpspt(pv[1].par_va.lit.str_va));
  }

/********************************************************/
/*!******************************************************/

        short evpopt(PMPARVA *pv)

/*      Evaluerar proceduren POP_PMT.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 10/4/87 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXpopt());
  }

/********************************************************/
/*!******************************************************/

        short evcrts(PMPARVA *pv)

/*      Evaluerar proceduren CRE_TSTR.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXcrts((short)pv[1].par_va.lit.int_va,
                  pv[2].par_va.lit.str_va));
  }

/********************************************************/
/*!******************************************************/

        short evcrmy(PMPARVA *pv, short npar)

/*      Evaluerar proceduren CRE_MEN.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          npar => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXcrmy(pv,npar));
  }

/********************************************************/
/*!******************************************************/

        short evpsmy(PMPARVA *pv)

/*      Evaluerar proceduren PSH_MEN.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXpsmy((short)pv[1].par_va.lit.int_va));
  }

/********************************************************/
/*!******************************************************/

        short evpomy(PMPARVA *pv)

/*      Evaluerar proceduren POP_MEN.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 10/4/87 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXpomy());
  }

/********************************************************/
/*!******************************************************/

        short evgtmy(PMPARVA *pv)

/*      Evaluerar proceduren GET_MEN.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87 R. Svedin
 *
 *      24/4/87  Felhantering, R. Svedin
 *
 ******************************************************!*/

  {

    short    status;
    PMLITVA  str;

    if ( (status=EXgtmy((short)pv[1].par_va.lit.int_va,
                 str.lit.str_va)) < 0 ) return(status);

/*
***Returnera parameter 1 = rubriktext.
*/
    evwval(&str, 1, pv);

    return(0);

  }

/********************************************************/
/*!******************************************************/

        short evlmdf(PMPARVA *pv)

/*      Evaluerar proceduren LOAD_MDF.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      Felkod: IN5252 = Menyfilen %s finns ej 
 *
 *      (C)microform ab 1996-05-21 J.Kjellander
 *
 ******************************************************!*/

  {
    if ( iglmdf(pv[1].par_va.lit.str_va) < 0 )
      return(erpush("IN5252",pv[1].par_va.lit.str_va));
    else
      return(0);
  }

/********************************************************/
/*!******************************************************/

        short evexst(PMLITVA *valp)

/*      Evaluerar funktionen EXESTAT.
 *
 *      In: valp => Pekare till resulat.
 *
 *      Ut: *valp.
 *
 *          0 = Kr aktiv modul, f5.
 *          1 = Skapa part, actfun=-2.
 *          2 = ndra part, f7.
 *          3 = Uppdatera part, f109.
 *
 *      FV: 0.
 *
 *      (C)microform ab 23/7/90 J. Kjellander
 *
 *      1996-05-28 f170, J.Kjellander
 *
 ******************************************************!*/

  {
    extern short actfun;

    switch (actfun)
      {
      case 5:
      valp->lit.int_va = 0;
      break;

      case -2:
      valp->lit.int_va = 1;
      break;

      case 7:
      case 170:
      valp->lit.int_va = 2;
      break;

      case 109:
      valp->lit.int_va = 3;
      break;

      default:
      valp->lit.int_va = 999;
      }

    return(0);

  }

/********************************************************/
