/**********************************************************************
*
*    eval7.c
*    =======
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evdbop();     Evaluerar DB_OPEN
*    evdbbg();     Evaluerar DB_BEGIN
*    evdbsl();     Evaluerar DB_SELECT
*    evdbup();     Evaluerar DB_UPDATE
*    evdbin();     Evaluerar DB_INSERT
*    evdbdl();     Evaluerar DB_DELETE
*    evdben();     Evaluerar DB_END
*    evdbrl();     Evaluerar DB_ROLLBACK
*    evdbcl();     Evaluerar DB_CLOSE
*    evdbfi();     Evaluerar DB_FETCHI
*    evdbff();     Evaluerar DB_FETCHF
*    evdbfs();     Evaluerar DB_FETCHS
*    evdbnx();     Evaluerar DB_NEXT
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"

/*!******************************************************/

        short evdbop(PMPARVA *pv, PMLITVA *valp)

/*      Evaluerar funktionen DB_OPEN.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbop(pv[1].par_va.lit.str_va,&valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbbg(PMPARVA *pv, PMLITVA *valp)

/*      Evaluerar funktionen DB_BEGIN.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbbg(pv[1].par_va.lit.str_va,&valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbsl(PMPARVA *pv, PMLITVA *valp, short npar)

/*      Evaluerar funktionen DB_SELECT.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *          npar  => Antal parametrar
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbsl(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.str_va,
                 &pv[3].par_va,
                  pv[4].par_va.lit.str_va,
                 &pv[5].par_va,
                 &pv[6],
           (short)(npar-5)/2,
                 &valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbup(PMPARVA *pv, PMLITVA *valp, short npar)

/*      Evaluerar funktionen DB_UPDATE.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *          npar  => Antal parametrar
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbup(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.str_va,
                 &pv[3].par_va,
                  pv[4].par_va.lit.str_va,
                 &pv[5].par_va,
                 &pv[6],
          (short)(npar-5)/2,
                 &valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbin(PMPARVA *pv, PMLITVA *valp, short npar)

/*      Evaluerar funktionen DB_INSERT.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *          npar  => Antal parametrar
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbin(pv[1].par_va.lit.str_va,
                 &pv[2],
          (short)(npar-1)/2,
                 &valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbdl(PMPARVA *pv, PMLITVA *valp)

/*      Evaluerar funktionen DB_DELETE.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbdl(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.str_va,
                 &pv[3].par_va,
                  pv[4].par_va.lit.str_va,
                 &pv[5].par_va,
                 &valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdben(PMLITVA *valp)

/*      Evaluerar funktionen DB_END.
 *
 *      In: valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdben(&valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbrl(PMLITVA *valp)

/*      Evaluerar funktionen DB_ROLLBACK.
 *
 *      In: valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbrl(&valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbcl(PMLITVA *valp)

/*      Evaluerar funktionen DB_CLOSE.
 *
 *      In: valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXdbcl(&valp->lit.int_va));

  }

/********************************************************/
/*!******************************************************/

        short evdbfi(PMPARVA *pv, PMLITVA *valp)

/*      Evaluerar funktionen DB_FETCHI.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {
    short   status;
    PMLITVA ival;

    if ( (status=EXdbfi((short)pv[1].par_va.lit.int_va,
          &ival.lit.int_va,&valp->lit.int_va)) < 0 ) return(status);

    else return(evwval(&ival, 1, pv));
  }

/********************************************************/
/*!******************************************************/

        short evdbff(PMPARVA *pv, PMLITVA *valp)

/*      Evaluerar funktionen DB_FETCHF.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {
    short   status;
    PMLITVA fval;

    if ( (status=EXdbff((short)pv[1].par_va.lit.int_va,
          &fval.lit.float_va,&valp->lit.int_va)) < 0 ) return(status);

    else return(evwval(&fval, 1, pv));
  }

/********************************************************/
/*!******************************************************/

        short evdbfs(PMPARVA *pv, PMLITVA *valp)

/*      Evaluerar funktionen DB_FETCHS.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {
    short   status;
    PMLITVA sval;

    if ( (status=EXdbfs((short)pv[1].par_va.lit.int_va,
          sval.lit.str_va,&valp->lit.int_va)) < 0 ) return(status);

    else return(evwval(&sval, 1, pv));
  }

/********************************************************/
/*!******************************************************/

        short evdbnx(PMLITVA *valp)

/*      Evaluerar funktionen DB_NEXT.
 *
 *      In: valp  => Pekare till resultat
 *
 *      Ut: *valp = Status frn databas.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 16/5/87 J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXdbnx(&valp->lit.int_va));
  }

/********************************************************/
