/**********************************************************************
*
*    eval9.c
*    =======
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evsexd();     Evaluerar SUR_EXDEF
*    evscon();     Evaluerar SUR_CONIC
*    evsoff();     Evaluerar SUR_OFFS
*    evssar();     Evaluerar SUR_SPLARR
*    evscar();     Evaluerar SUR_CONARR
*    evsnar();     Evaluerar SUR_NURBSARR
*    evsapp();     Evaluerar SUR_APPROX
*    evscom();     Evaluerar SUR_COMP
*    evstrm();     Evaluerar SUR_TRIM
*    evsrot();     Evaluerar SUR_ROT
*    evsusr();     Evaluerar SUR_USRDEF
*    evscyl();     Evaluerar SUR_CYL
*    evsswp();     Evaluerar SUR_SWEEP
*    evsrul();     Evaluerar SUR_RULED
*    evscur();     Evaluates SUR_CURVES
*
*    evcsil();     Evaluerar CUR_SIL
*    evcint();     Evaluerar CUR_INT
*    evciso();     Evaluerar CUR_ISO
*    evcfan();     Evaluerar CUR_FANGA
*    evcged();     Evaluerar CUR_GEODESIC
*    evcapp();     Evaluerar CUR_APPROX
*    evcugl();     Evaluerar CUR_GL
*
*    evions();     Evaluerar INV_ONSUR
*    eveval();     Evaluerar EVAL
*    evncui();     Evaluerar N_CUR_INT
*    evails();     Evaluerar ALL_INT_LIN_SUR
*    evailm();     Evaluerar ALL_INT_LIN_MSUR
*    evaios();     Evaluerar ALL_INV_ON_SUR
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"

/*!******************************************************/

        short evsexd(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_EXDEF.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 20/11/92 J. Kjellander
 *
 *      1997-03-12 pv[8], J.Kjellander
 *
 ******************************************************!*/

{
    gmflt trim[4];
/*
***trim-vrden.
*/
   trim[0] = pv[4].par_va.lit.float_va;
   trim[1] = pv[5].par_va.lit.float_va;
   trim[2] = pv[6].par_va.lit.float_va;
   trim[3] = pv[7].par_va.lit.float_va;
/*
***Exekvera SUR_EXDEF.
*/
   return(EXsexd( ident, pv[1].par_va.lit.str_va,
                         pv[2].par_va.lit.str_va,
                         pv[3].par_va.lit.str_va,
                         trim,
                    (int)pv[8].par_va.lit.int_va,
                         npblockp));
}

/********************************************************/
/*!******************************************************/

        short evscon(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren SUR_CONIC.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      Ut: Inget.
 *
 *      Felkkoder: IN5082 = Typ ej m/M/p/P
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 25/4/94 J. Kjellander
 *
 ******************************************************!*/

{
   short    status;
   DBint    i,j,nstrips,typarr[MXLSTP];
   V2REFVA *spine;
   V2REFVA *limarr[MXLSTP];
   V2REFVA *tanarr[MXLSTP];
   V2REFVA *midarr[MXLSTP];

/*
***ID fr spinekurva.
*/
   spine = &pv[1].par_va.lit.ref_va[0];
/*
***Hur mnga strippar ?
*/
   nstrips = (np - 3)/4;
/*
***Skapa limarr och tanarr.
*/
   for( j=0,i=2; j<=nstrips; j++,i+=4 )
     { 
     limarr[j] = &pv[i  ].par_va.lit.ref_va[0];
     tanarr[j] = &pv[i+1].par_va.lit.ref_va[0];
     }
/*
***Skapa typarr och midarr.
*/
   for( j=0,i=4; j<nstrips; j++,i+=4 )
     { 
     if      ( sticmp(pv[i].par_va.lit.str_va,"p") == 0 ) typarr[j] = 1;
     else if ( sticmp(pv[i].par_va.lit.str_va,"m") == 0 ) typarr[j] = 2;
     else      return(erpush("IN5082",pv[i].par_va.lit.str_va));
     midarr[j] = &pv[i+1].par_va.lit.ref_va[0];
     }
/*
***Exekvera SUR_CONIC.
*/
    status = EXscon(ident,spine,nstrips,
                       limarr,tanarr,typarr,midarr,npblockp);

   return(status);
}

/********************************************************/
/*!******************************************************/

        short evsoff(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_OFFS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 25/4/94 J. Kjellander
 *
 ******************************************************!*/

{

/*
***Exekvera SUR_OFFS.
*/
   return(EXsoff( ident, pv[1].par_va.lit.ref_va,
                         pv[2].par_va.lit.float_va,
                         npblockp));
}

/********************************************************/
/*!******************************************************/

        short evssar(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren SUR_SPLARR.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      Ut: Inget.
 *
 *      Felkkoder: IN5102 = Kan ej allokera minne
 *                 IN5112 = Orimligt antal positioner.
 *                 IN5122 = nu eller nv > deklarerat
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 *      3/4/95  tv,tw, J. Kjellander
 *
 ******************************************************!*/

 {
   char    errbuf[81];
   short   status;
   int     nu_dekl,nv_dekl,dekl_dim,vecsiz;
   gmint   nu,nv,i,j;
   DBint   posadr,tuadr=0,tvadr=0,twadr=0;
   STTYTBL typtbl;
   STARRTY arrtyp;
   DBVector  *ppek,*tupek,*tvpek,*twpek;
   PMLITVA val;

/*
***Hur mnga positioner.
*/
   nu = pv[2].par_va.lit.int_va;
   nv = pv[3].par_va.lit.int_va;
/*
***Kolla att antalet positioner r rimligt.
*/
   if ( nu < 2 ) return(erpush("IN5112","U"));
   if ( nv < 2 ) return(erpush("IN5112","V"));
/*
***Kolla att deklarerad dimension p MBS-arrayer r tillrcklig fr
***nu*nv positioner och tangenter.
*/
   posadr = pv[4].par_va.lit.adr_va;
/*
***Strtyp kan alltid gras p en variabel och en variabel vet vi
***skert att det r eftersom v3c kollar detta (.av. i formpar.h).
*/
   strtyp(pv[4].par_ty,&typtbl);
/*
***typtbl r en structure som bland annat talar om hurvida variabeln r
***en array eller inte. Om inte, felmeddelande.
*/
   if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","no array"));
/*
***Om det r en array kan vi nu ta reda p vilken slags array.
*/
   strarr(typtbl.arr_ty,&arrtyp);
/*
***arrtyp r en structure som bland annat talar om hur den r dimensionerad.
***Den skall minst vara lika stor som antalet punkter i U-riktningen.
*/
   nu_dekl = arrtyp.up_arr - arrtyp.low_arr + 1;
   if ( nu_dekl < nu ) return(erpush("IN5122","nu > decl"));
/*
***Denna array skall i sin tur best av arrayer.
*/
   strtyp(arrtyp.base_arr,&typtbl);
   if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","1-dim array"));
/*
***Som ocks skall vara tillrckligt stora.
***Dvs. minst lika stora som antalet punkter i V-riktningen.
*/
   strarr(typtbl.arr_ty,&arrtyp);
   nv_dekl = arrtyp.up_arr - arrtyp.low_arr + 1;
   if ( nv_dekl < nv ) return(erpush("IN5122","nv > decl"));
/*
***Allokera minne fr positioner.
*/
   if ( (ppek=(DBVector *)v3mall(nu*nv*sizeof(DBVector),"evssar")) == NULL )
     {
     sprintf(errbuf,"%d",(int)(nu*nv));
     return(erpush("IN5102",errbuf));
     }
/*
***Likas U-tangenterna.
*/
   if ( np > 4 )
     {
     tuadr = pv[5].par_va.lit.adr_va;
     strtyp(pv[5].par_ty,&typtbl);

     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","no array"));
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim != nu_dekl ) return(erpush("IN5122","tu(nu,-) != p(nu,-)"));
     strtyp(arrtyp.base_arr,&typtbl);
     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","1-dim array"));
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim != nv_dekl ) return(erpush("IN5122","tu(-,nv) != p(-,nv)"));

     if ( (tupek=(DBVector *)v3mall(nu*nv*sizeof(DBVector),"evssar")) == NULL )
       {
       sprintf(errbuf,"%d",(int)(nu*nv));
       return(erpush("IN5102",errbuf));
       }
     }
   else tupek = NULL;
/*
***V-tangenterna.
*/
   if ( np > 5 )
     {
     tvadr = pv[6].par_va.lit.adr_va;
     strtyp(pv[6].par_ty,&typtbl);

     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","no array"));
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim != nu_dekl ) return(erpush("IN5122","tv(nu,-) != p(nu,-)"));
     strtyp(arrtyp.base_arr,&typtbl);
     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","1-dim array"));
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim != nv_dekl ) return(erpush("IN5122","tv(-,nv) != p(-,nv)"));

     if ( (tvpek=(DBVector *)v3mall(nu*nv*sizeof(DBVector),"evssar")) == NULL )
       {
       sprintf(errbuf,"%d",(int)(nu*nv));
       return(erpush("IN5102",errbuf));
       }
     }
   else tvpek = NULL;
/*
***Och twist.
*/
   if ( np > 6 )
     {
     twadr = pv[7].par_va.lit.adr_va;
     strtyp(pv[7].par_ty,&typtbl);

     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","no array"));
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim != nu_dekl ) return(erpush("IN5122","tw(nu,-) != p(nu,-)"));
     strtyp(arrtyp.base_arr,&typtbl);
     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5122","1-dim array"));
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim != nv_dekl ) return(erpush("IN5122","tw(-,nv) != p(-,nv)"));

     if ( (twpek=(DBVector *)v3mall(nu*nv*sizeof(DBVector),"evssar")) == NULL )
       {
       sprintf(errbuf,"%d",(int)(nu*nv));
       return(erpush("IN5102",errbuf));
       }
     }
   else twpek = NULL;
/*
***vecsiz = storleken p en VECTOR.
*/
   strtyp(arrtyp.base_arr,&typtbl);
   vecsiz = typtbl.size_ty;
/*
***Kopiera positioner frn RTS till allokerad area.
*/
   for ( i=0; i<nu; ++i )
     {
     for ( j=0; j<nv; ++j )
       {
       ingval(posadr+(i*nv_dekl+j)*vecsiz,arrtyp.base_arr,FALSE,&val);
       (ppek+i*nv+j)->x_gm = val.lit.vec_va.x_val;
       (ppek+i*nv+j)->y_gm = val.lit.vec_va.y_val;
       (ppek+i*nv+j)->z_gm = val.lit.vec_va.z_val;
       if ( np > 4 )
         {
         ingval(tuadr+(i*nv_dekl+j)*vecsiz,arrtyp.base_arr,FALSE,&val);
         (tupek+i*nv+j)->x_gm = val.lit.vec_va.x_val;
         (tupek+i*nv+j)->y_gm = val.lit.vec_va.y_val;
         (tupek+i*nv+j)->z_gm = val.lit.vec_va.z_val;
         }
       if ( np > 5 )
         {
         ingval(tvadr+(i*nv_dekl+j)*vecsiz,arrtyp.base_arr,FALSE,&val);
         (tvpek+i*nv+j)->x_gm = val.lit.vec_va.x_val;
         (tvpek+i*nv+j)->y_gm = val.lit.vec_va.y_val;
         (tvpek+i*nv+j)->z_gm = val.lit.vec_va.z_val;
         }
       if ( np > 6 )
         {
         ingval(twadr+(i*nv_dekl+j)*vecsiz,arrtyp.base_arr,FALSE,&val);
         (twpek+i*nv+j)->x_gm = val.lit.vec_va.x_val;
         (twpek+i*nv+j)->y_gm = val.lit.vec_va.y_val;
         (twpek+i*nv+j)->z_gm = val.lit.vec_va.z_val;
         }
       }
     } 
/*
***Anropa exe-rutinen.
*/
   status = EXssar(ident,pv[1].par_va.lit.str_va,nu,nv,
                         ppek,tupek,tvpek,twpek,npblockp);
/*
***terlmna allokerat minne.
*/
   v3free(ppek,"evssar");
   if ( tupek != NULL ) v3free(tupek,"evssar");
   if ( tvpek != NULL ) v3free(tvpek,"evssar");
   if ( twpek != NULL ) v3free(twpek,"evssar");

   return(status);
 }

/********************************************************/
/*!******************************************************/

        short evscar(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_CONARR.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5202 = Antal kurvor < 2.
 *                IN5212 = Antal kurvor > deklarerad storlek.
 *                IN5222 = Kan ej allokera minne.
 *                IN5232 = Typ skall vara m/M/p/P.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 *      3/4/95  metod, J. Kjellander
 *
 ******************************************************!*/

{
   char     errbuf[81];
   short    status;
   int      ncur,dekl_dim,refsiz,strsiz;
   gmint    i,*metpek;
   DBint    limadr,tanadr,metadr,mpadr;
   STTYTBL  typtbl;
   STARRTY  limtyp,tantyp,mettyp,mptyp;
   PMREFVA *lpek,*tpek,*mppek;
   PMLITVA  val;

/*
***Hur mnga kurvor.
*/
   ncur = pv[2].par_va.lit.int_va;
/*
***Kolla att antalet r rimligt.
*/
   if ( ncur < 2 ) return(erpush("IN5202",""));
/*
***Kolla att deklarerad dimension p MBS-arrayer r tillrcklig fr
***ncur kurvor etc.
*/
   limadr = pv[4].par_va.lit.adr_va;
   strtyp(pv[4].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&limtyp);
   dekl_dim = limtyp.up_arr - limtyp.low_arr + 1;
   if ( dekl_dim < ncur ) return(erpush("IN5212",""));

   strtyp(limtyp.base_arr,&typtbl);
   refsiz = typtbl.size_ty;

   tanadr = pv[5].par_va.lit.adr_va;
   strtyp(pv[5].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&tantyp);
   dekl_dim = tantyp.up_arr - tantyp.low_arr + 1;
   if ( dekl_dim < ncur ) return(erpush("IN5212",""));

   metadr = pv[6].par_va.lit.adr_va;
   strtyp(pv[6].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&mettyp);
   dekl_dim = mettyp.up_arr - mettyp.low_arr + 1;
   if ( dekl_dim < ncur-1 ) return(erpush("IN5212",""));

   strtyp(mettyp.base_arr,&typtbl);
   strsiz = typtbl.size_ty;

   mpadr = pv[7].par_va.lit.adr_va;
   strtyp(pv[7].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&mptyp);
   dekl_dim = mptyp.up_arr - mptyp.low_arr + 1;
   if ( dekl_dim < ncur-1 ) return(erpush("IN5212",""));
/*
***Allokera minne.
*/
   if ( (lpek=(PMREFVA *)v3mall(ncur*sizeof(PMREFVA),"evscar")) == NULL )
     {
     sprintf(errbuf,"%d",ncur);
     return(erpush("IN5222",errbuf));
     }

   if ( (tpek=(PMREFVA *)v3mall(ncur*sizeof(PMREFVA),"evscar")) == NULL )
     {
     sprintf(errbuf,"%d",ncur);
     return(erpush("IN5222",errbuf));
     }

   if ( (metpek=(gmint *)v3mall(ncur*sizeof(gmint),"evscar")) == NULL )
     {
     sprintf(errbuf,"%d",ncur);
     return(erpush("IN5222",errbuf));
     }

   if ( (mppek=(PMREFVA *)v3mall(ncur*sizeof(PMREFVA),"evscar")) == NULL )
     {
     sprintf(errbuf,"%d",ncur);
     return(erpush("IN5222",errbuf));
     }
/*
***Kopiera frn RTS till allokerade areor.
*/
   for ( i=0; i<ncur; ++i )
     {
     ingval(limadr+i*refsiz,limtyp.base_arr,FALSE,&val);
     (lpek+i)->seq_val  = val.lit.ref_va[0].seq_val;
     (lpek+i)->ord_val  = val.lit.ref_va[0].ord_val;
     (lpek+i)->p_nextre = val.lit.ref_va[0].p_nextre;

     ingval(tanadr+i*refsiz,tantyp.base_arr,FALSE,&val);
     (tpek+i)->seq_val  = val.lit.ref_va[0].seq_val;
     (tpek+i)->ord_val  = val.lit.ref_va[0].ord_val;
     (tpek+i)->p_nextre = val.lit.ref_va[0].p_nextre;
     }

   for ( i=0; i<ncur-1; ++i )
     {
     ingval(metadr+i*strsiz,mettyp.base_arr,FALSE,&val);
     if ( sticmp(val.lit.str_va,"p") == 0 )
       {
      *(metpek+i) = 1;
       }
     else if ( sticmp(val.lit.str_va,"m") == 0 )
       {
      *(metpek+i) = 2;
       }
     else
       {
       status = erpush("IN5232","");
       goto exit;
       }
     ingval(mpadr+i*refsiz,mptyp.base_arr,FALSE,&val);
     (mppek+i)->seq_val  = val.lit.ref_va[0].seq_val;
     (mppek+i)->ord_val  = val.lit.ref_va[0].ord_val;
     (mppek+i)->p_nextre = val.lit.ref_va[0].p_nextre;
     }
/*
***Anropa exe-rutinen.
*/
   status = EXscar(ident,pv[1].par_va.lit.str_va,
                        &pv[3].par_va.lit.ref_va[0],
                  (gmint)ncur,
                         lpek,tpek,metpek,mppek,npblockp);
/*
***Deallokera minne.
*/
exit:
   v3free(lpek,"evscar");
   v3free(tpek,"evscar");
   v3free(metpek,"evscar");
   v3free(mppek,"evscar");

   return(status);
}

/********************************************************/
/*!******************************************************/

        short evsnar(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren SUR_NURBSARR.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5222 = Kan ej allokera minne.
 *                IN5372 = cpts/w felaktigt deklarerad
 *                IN5402 = cpts fel typ
 *                IN5382 = uknots/vknots fr liten
 *                IN5392 = w fr liten
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-12-17 J. Kjellander
 *
 ******************************************************!*/

{
   short     status;
   int       dekl_dim,dekl_dim_v1,dekl_dim_v2=0,i,j,
             fltsiz=0,vecsiz,np_u,np_v;
   DBint     order_u,order_v,nk_u,nk_v;
   DBint     posadr,ukadr=0,vkadr=0,wadr=0;
   DBfloat  *ukpek,*vkpek,*wpek;
   DBVector *cppek;
   STTYTBL   typtbl;
   STARRTY   arrtyp1,arrtyp2,arrtyp3,arrtyp4;
   PMLITVA   val;

/*
***Ordning i U och V.
*/
   order_u = pv[1].par_va.lit.int_va;
   order_v = pv[2].par_va.lit.int_va;
/*
***Antal knutpunker i U och V.
*/
   nk_u = pv[3].par_va.lit.int_va;
   nk_v = pv[4].par_va.lit.int_va;
/*
***Antal styrpunkter i U och V.
*/
   np_u = nk_u - order_u;
   np_v = nk_v - order_v;
/*
***Kolla att deklarerad dimension p cpts r tillrcklig.
*/
   posadr = pv[5].par_va.lit.adr_va;
   strtyp(pv[5].par_ty,&typtbl);
   if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5372","no array"));
   strarr(typtbl.arr_ty,&arrtyp1);
   dekl_dim = arrtyp1.up_arr - arrtyp1.low_arr + 1;
   if ( dekl_dim < np_u ) return(erpush("IN5372","u-size too small"));

   strtyp(arrtyp1.base_arr,&typtbl);
   if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5372","1-dim array"));
   strarr(typtbl.arr_ty,&arrtyp1);
   dekl_dim_v1 = arrtyp1.up_arr - arrtyp1.low_arr + 1;
   if ( dekl_dim_v1 < np_v ) return(erpush("IN5372","v-size too small"));

   strtyp(arrtyp1.base_arr,&typtbl);
   if ( typtbl.kind_ty != ST_VEC ) return(erpush("IN5402",""));
   vecsiz = typtbl.size_ty;
/*
***Kolla att deklarerad dimension p ev. knots r ok.
*/
   if ( np > 5 )
     {
     ukadr = pv[6].par_va.lit.adr_va;
     strtyp(pv[6].par_ty,&typtbl);
     strarr(typtbl.arr_ty,&arrtyp2);
     dekl_dim = arrtyp2.up_arr - arrtyp2.low_arr + 1;
     if ( dekl_dim < nk_u ) return(erpush("IN5382",""));

     strtyp(arrtyp2.base_arr,&typtbl);
     fltsiz = typtbl.size_ty;
     }

   if ( np > 6 )
     {
     vkadr = pv[7].par_va.lit.adr_va;
     strtyp(pv[7].par_ty,&typtbl);
     strarr(typtbl.arr_ty,&arrtyp3);
     dekl_dim = arrtyp3.up_arr - arrtyp3.low_arr + 1;
     if ( dekl_dim < nk_v ) return(erpush("IN5382",""));
     }
/*
***Kolla typ och dimension p ev. w.
*/
   if ( np > 7 )
     {
     wadr = pv[8].par_va.lit.adr_va;
     strtyp(pv[8].par_ty,&typtbl);
     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5372","no array"));
     strarr(typtbl.arr_ty,&arrtyp4);
     dekl_dim = arrtyp4.up_arr - arrtyp4.low_arr + 1;
     if ( dekl_dim < np_u ) return(erpush("IN5372","u-size too small"));

     strtyp(arrtyp4.base_arr,&typtbl);
     if ( typtbl.arr_ty == (pm_ptr)NULL ) return(erpush("IN5372","1-dim array"));
     strarr(typtbl.arr_ty,&arrtyp4);
     dekl_dim_v2 = arrtyp4.up_arr - arrtyp4.low_arr + 1;
     if ( dekl_dim_v2 < np_v ) return(erpush("IN5372","v-size too small"));

     strtyp(arrtyp4.base_arr,&typtbl);
     if ( typtbl.kind_ty != ST_FLOAT ) return(erpush("IN5392",""));
     }
/*
***Allokera minne.
*/
   if ( (cppek=(DBVector *)v3mall(np_u*np_v*sizeof(DBVector),"evsnar")) == NULL )
     return(erpush("IN5412",""));

   if ( np > 5 )
     {
     if ( (ukpek=(gmflt *)v3mall(nk_u*sizeof(gmflt),"evsnar")) == NULL )
       return(erpush("IN5412",""));
     }
   else ukpek = NULL;

   if ( np > 6 )
     {
     if ( (vkpek=(gmflt *)v3mall(nk_v*sizeof(gmflt),"evsnar")) == NULL )
       return(erpush("IN5412",""));
     }
   else vkpek = NULL;

   if ( np > 7 )
     {
     if ( (wpek=(gmflt *)v3mall(np_u*np_v*sizeof(gmflt),"evsnar")) == NULL )
       return(erpush("IN5412",""));
     }
   else wpek = NULL;
/*
***Kopiera frn RTS till allokerade areor.
*/
   for ( i=0; i<np_u; ++i )
     {
     for ( j=0; j<np_v; ++j )
       {
       ingval(posadr+(i*dekl_dim_v1+j)*vecsiz,arrtyp1.base_arr,FALSE,&val);
       (cppek+i*np_v+j)->x_gm = val.lit.vec_va.x_val;
       (cppek+i*np_v+j)->y_gm = val.lit.vec_va.y_val;
       (cppek+i*np_v+j)->z_gm = val.lit.vec_va.z_val;
       }
     } 

   if ( np > 5 )
     {
     for ( i=0; i<nk_u; ++i )
       {
       ingval(ukadr+i*fltsiz,arrtyp2.base_arr,FALSE,&val);
      *(ukpek+i) = val.lit.float_va;
       }
     }

   if ( np > 6 )
     {
     for ( i=0; i<nk_v; ++i )
       {
       ingval(vkadr+i*fltsiz,arrtyp3.base_arr,FALSE,&val);
      *(vkpek+i) = val.lit.float_va;
       }
     }

   if ( np > 7 )
     {
     for ( i=0; i<np_u; ++i )
       {
       for ( j=0; j<np_v; ++j )
         {
         ingval(wadr+(i*dekl_dim_v2+j)*fltsiz,arrtyp4.base_arr,FALSE,&val);
        *(wpek+i*np_v+j) = val.lit.float_va;
         }
       } 
     }
/*
***Anropa exe-rutinen.
*/
   status = EXsnar(ident,pv[1].par_va.lit.int_va,
                         pv[2].par_va.lit.int_va,
                         pv[3].par_va.lit.int_va,
                         pv[4].par_va.lit.int_va,
                         cppek,ukpek,vkpek,wpek,npblockp);
/*
***Deallokera minne.
*/
   v3free(ukpek,"evsnar");
   v3free(vkpek,"evsnar");
   v3free(cppek,"evsnar");

   return(status);
}

/********************************************************/
/*!******************************************************/

        short evsapp(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren SUR_APPROX.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
   gmflt tol_2;

/*
***Optionell parameter.
*/
   if ( np == 5 ) tol_2 = pv[5].par_va.lit.float_va;
   else           tol_2 = pv[4].par_va.lit.float_va;
/*
***Exekvera.
*/
   return(EXsapp(ident,pv[1].par_va.lit.ref_va,
                       pv[2].par_va.lit.str_va,
                       pv[3].par_va.lit.int_va,
                       pv[4].par_va.lit.float_va,
                       tol_2,
                       npblockp));
}

/********************************************************/
/*!******************************************************/

        short evscom(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    parcount)

/*      Evaluerar geometri-proceduren SUR_COMP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          parcount => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
   int     i;
   int     nref = 0;
   V2REFVA refarr[GMMXCO+1];

/*
***Skapa refarr.
*/
   for ( i=1; i<=parcount; i++ )
     {
     refarr[nref].seq_val  = pv[i].par_va.lit.ref_va[0].seq_val;
     refarr[nref].ord_val  = pv[i].par_va.lit.ref_va[0].ord_val;
     refarr[nref].p_nextre = pv[i].par_va.lit.ref_va[0].p_nextre;
     nref++;
     }
/*
***Exekvera.
*/
   return(EXscom(ident,refarr,nref,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evstrm(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren SUR_TRIM.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
   int fall;

/*
***Optionell parameter fall.
*/
   if ( np == 6 ) fall = pv[6].par_va.lit.int_va;
   else           fall = 1;
/*
***Exekvera.
*/
   return(EXstrm(ident,&pv[1].par_va.lit.ref_va[0],
                        pv[2].par_va.lit.float_va,
                        pv[3].par_va.lit.float_va,
                        pv[4].par_va.lit.float_va,
                        pv[5].par_va.lit.float_va,
                 (gmint)fall,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evsrot(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_ROT.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1996-12-10 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
   return(EXsrot(ident,&pv[1].par_va.lit.ref_va[0],
                       (DBVector *)&pv[2].par_va.lit.vec_va,
                       (DBVector *)&pv[3].par_va.lit.vec_va,
                        pv[4].par_va.lit.float_va,
                        pv[5].par_va.lit.float_va,
                        pv[6].par_va.lit.int_va,
                        pv[7].par_va.lit.int_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evsusr(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_USRDEF.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-03-27 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
   return(EXsusr(ident,&pv[1].par_va.lit.ref_va[0],
                        pv[2].par_va.lit.str_va,
                        pv[3].par_va.lit.str_va,
                        pv[4].par_va.lit.str_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evsswp(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_SWEEP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-03-27 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
   return(EXsswp(ident,&pv[1].par_va.lit.ref_va[0],
                       &pv[2].par_va.lit.ref_va[0],
                        pv[3].par_va.lit.int_va,
                       (DBVector *)&pv[4].par_va.lit.vec_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evscyl(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_CYL.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-03-27 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
   return(EXscyl(ident,&pv[1].par_va.lit.ref_va[0],
                        pv[2].par_va.lit.float_va,
                        pv[3].par_va.lit.int_va,
                       (DBVector *)&pv[4].par_va.lit.vec_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evsrul(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SUR_CYL.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      Felkkoder:
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-03-27 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
   return(EXsrul(ident,&pv[1].par_va.lit.ref_va[0],
                       &pv[2].par_va.lit.ref_va[0],
                        pv[3].par_va.lit.int_va,
                        pv[4].par_va.lit.int_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcsil(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_SIL.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 20/11/92 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera CUR_SIL.
*/
   return(EXcsil( ident,&pv[1].par_va.lit.ref_va[0],
                        (DBVector *)&pv[2].par_va.lit.vec_va,
                  (short)pv[3].par_va.lit.int_va,
                  (short)pv[4].par_va.lit.int_va, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcint(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_INT.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 20/11/92 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera CUR_INT.
*/
   return(EXcint( ident,&pv[1].par_va.lit.ref_va[0],
                        &pv[2].par_va.lit.ref_va[0],
                  (short)pv[3].par_va.lit.int_va, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evciso(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_ISO.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 20/11/92 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera CUR_ISO.
*/
   return(EXciso( ident,&pv[1].par_va.lit.ref_va[0],
                         pv[2].par_va.lit.str_va,
                         pv[3].par_va.lit.float_va,
                         pv[4].par_va.lit.float_va,
                         pv[5].par_va.lit.float_va, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcfan(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren CUR_FANGA.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
   gmint sstart,sslut;
   gmflt ustart,uslut;

/*
***Optionella parametrar.
*/
   if ( np == 7 )
     {
     sstart = pv[4].par_va.lit.int_va;
     sslut  = pv[5].par_va.lit.int_va;
     ustart = pv[6].par_va.lit.float_va;
     uslut  = pv[7].par_va.lit.float_va;
     }
   else
     {
     sstart = 0;
     sslut  = 0;
     ustart = 0.0;
     uslut  = 0.0;
     }
/*
***Exekvera.
*/
   return(EXcfan( ident,&pv[1].par_va.lit.ref_va[0],
                        (DBVector *)&pv[2].par_va.lit.vec_va,
                        (DBVector *)&pv[3].par_va.lit.vec_va,
                         sstart,sslut,ustart,uslut,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcged(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_GEODESIC.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
   return(EXcged( ident,&pv[1].par_va.lit.ref_va[0],
                         pv[2].par_va.lit.str_va,
                        (DBVector *)&pv[3].par_va.lit.vec_va,
                        (DBVector *)&pv[4].par_va.lit.vec_va,
                         npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcapp(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren CUR_APPROX.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
   gmflt tol_2;

/*
***Optionell parameter.
*/
   if ( np == 5 ) tol_2 = pv[5].par_va.lit.float_va;
   else           tol_2 = pv[4].par_va.lit.float_va;
/*
***Exekvera.
*/
   return(EXcapp( ident,&pv[1].par_va.lit.ref_va[0],
                         pv[2].par_va.lit.str_va,
                         pv[3].par_va.lit.int_va,
                         pv[4].par_va.lit.float_va,
                         tol_2,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcugl(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_GL.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 20/11/92 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera CUR_GL.
*/
   return(EXcugl( ident,&pv[1].par_va.lit.ref_va[0],
                         pv[2].par_va.lit.str_va, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evions(
        PMPARVA  pv[],
        PMLITVA *valp,
        short    np)

/*      Evaluerar funktionen INV_ON_SUR.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till blivande resultat.
 *          np   => Antal parametrar, 4 eller 5.
 *
 *      Ut: *valp = Funktionsvrdet.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 25/4/94 J. Kjellander
 *
 ******************************************************!*/

{
    DBVector uv_start;

/*
***Har startpunkt getts ? Om inte skicka (0,0,-1),
***detta tolkas av surpac-rutinen som "hela ytan".
*/
    if ( np == 5 )
      {
      uv_start.x_gm = pv[5].par_va.lit.vec_va.x_val;
      uv_start.y_gm = pv[5].par_va.lit.vec_va.y_val;
      uv_start.z_gm = 0;
      }
    else
      {
      uv_start.x_gm =  0;
      uv_start.y_gm =  0;
      uv_start.z_gm = -1;
      }
/*
***Exekvera.
*/
    return(EXions(&pv[1].par_va.lit.ref_va[0],
         (DBVector *)&pv[2].par_va.lit.vec_va,
            (gmint)pv[3].par_va.lit.int_va,
            (gmint)pv[4].par_va.lit.int_va,
                  &uv_start,
         (DBVector *)&valp->lit.vec_va));
}

/********************************************************/
/*!******************************************************/

        short eveval(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluerar funktionen EVAL.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till blivande resultat.
 *
 *      Ut: *valp = Funktionsvrdet.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 25/4/94 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera.
*/
    return(EXeval(&pv[1].par_va.lit.ref_va[0],
                   pv[2].par_va.lit.str_va,
         (DBVector *)&pv[3].par_va.lit.vec_va,
         (DBVector *)&valp->lit.vec_va));
}

/********************************************************/
/*!******************************************************/

        short evncui(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluerar funktionen N_CUR_INT.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till blivande resultat.
 *
 *      Ut: *valp = Funktionsvrdet.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 25/4/94 J. Kjellander
 *
 ******************************************************!*/

{
   return(EXncui(&pv[1].par_va.lit.ref_va[0],
                 &pv[2].par_va.lit.ref_va[0],
                 &valp->lit.int_va));
}

/********************************************************/
/*!******************************************************/

        short evails(
        PMPARVA  pv[],
        short    np)

/*      Evaluerar proceduren ALL_INT_LIN_SUR.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          np   => Antal parametrar.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 13/9/95 J. Kjellander
 *
 ******************************************************!*/

{
   short   status;
   gmint   nint;
   DBVector   uv[25],xyz[25];
   PMLITVA litval;

/*
***Exekvera.
*/
   status = EXails(&pv[1].par_va.lit.ref_va[0],
                   (DBVector *)&pv[2].par_va.lit.vec_va,
                   (DBVector *)&pv[3].par_va.lit.vec_va,
                    pv[4].par_va.lit.int_va,
                   (DBVector *)&pv[5].par_va.lit.vec_va,
                   &nint,uv,xyz);

   if ( status < 0 ) return(status);
/*
***Skriv in resultat i motsvarande MBS-variabler.
*/
   litval.lit.int_va = nint;
   inwvar(pv[6].par_ty,pv[6].par_va.lit.adr_va,0,NULL,&litval);

   if ( np > 6 )
     {
     if ( (status=evwvec(uv,(short)nint,(short)7,pv)) < 0 ) return(status);
     if ( np > 7 )
       {
       if ( (status=evwvec(xyz,(short)nint,(short)8,pv)) < 0 ) return(status);
       }
     }

   return(0);
}

/********************************************************/
/*!******************************************************/

        short evailm(
        PMPARVA  pv[],
        short    np)

/*      Evaluerar proceduren ALL_INT_LIN_MSUR.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          np   => Antal parametrar.
 *
 *      Felkoder: IN5442 = Fr f ytor
 *                IN5452 = Feldimensionerad in-array
 *                IN5462 = Can't malloc
 *                IN5472 = Feldimensionerad ut-array
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1998-04-24 J. Kjellander
 *
 ******************************************************!*/

{
   short    status;
   char     errbuf[V3STRLEN];
   DBint    nint,nref;
   DBint    i,dekl_dim,refsiz,*snumpt;
   DBint    refadr;
   DBVector uv[25],xyz[25];
   PMLITVA *rlitpt,litval;
   STTYTBL  typtbl;
   STARRTY  reftyp;
/*
***Antal ytor.
*/
   nref = pv[2].par_va.lit.int_va;
/*
***Kolla att antalet ytor r rimligt.
*/
   if ( nref < 1 ) return(erpush("IN5442",""));
/*
***Kolla att deklarerad dimension p refarr r tillrcklig.
***Att det r rtt typ kollas av mbsc.
*/
   refadr = pv[1].par_va.lit.adr_va;
   strtyp(pv[1].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&reftyp);
   dekl_dim = reftyp.up_arr - reftyp.low_arr + 1;
   if ( dekl_dim < nref ) return(erpush("IN5452",""));

   strtyp(reftyp.base_arr,&typtbl);
   refsiz = typtbl.size_ty;
/*
***Allokera minne fr refererade ytor.
*/
   if ( (rlitpt=(PMLITVA *)v3mall(nref*sizeof(PMLITVA),"evailm")) == NULL )
     {
     sprintf(errbuf,"%d",(int)nref);
     return(erpush("IN5462",errbuf));
     }
/*
***Kopiera frn RTS till allokerad area.
*/
   for ( i=0; i<nref; ++i )
     ingval(refadr+i*refsiz,reftyp.base_arr,FALSE,rlitpt+i);
/*
***Exekvera. EXailm() allokerar minne fr den array
***av PMREFVA som innehller Yt-ID fr trffarna.
*/
   status = EXailm(rlitpt,
                   pv[2].par_va.lit.int_va,
                  (DBVector *)&pv[3].par_va.lit.vec_va,
                  (DBVector *)&pv[4].par_va.lit.vec_va,
                   pv[5].par_va.lit.int_va,
                  &nint,uv,xyz,&snumpt);

   if ( status < 0 )
     {
     v3free(rlitpt,"evailm");
     return(status);
     }
/*
***Skriv in resultat i motsvarande MBS-variabler. Frst
***antal lsningar.
*/
   litval.lit.int_va = nint;
   inwvar(pv[6].par_ty,pv[6].par_va.lit.adr_va,0,NULL,&litval);
/*
***Sen UV-vrden.
*/
   if ( np > 6 )
     {
     refadr = pv[7].par_va.lit.adr_va;
     strtyp(pv[7].par_ty,&typtbl);
     strarr(typtbl.arr_ty,&reftyp);
     dekl_dim = reftyp.up_arr - reftyp.low_arr + 1;
     if ( dekl_dim < nint ) return(erpush("IN5452",""));
     if ( (status=evwvec(uv,(short)nint,(short)7,pv)) < 0 ) return(status);
/*
***Sen R3-vrden.
*/
     if ( np > 7 )
       {
       refadr = pv[8].par_va.lit.adr_va;
       strtyp(pv[8].par_ty,&typtbl);
       strarr(typtbl.arr_ty,&reftyp);
       dekl_dim = reftyp.up_arr - reftyp.low_arr + 1;
       if ( dekl_dim < nint ) return(erpush("IN5452",""));
       if ( (status=evwvec(xyz,(short)nint,(short)8,pv)) < 0 ) return(status);
/*
***Slutligen Yt-ID:n.
*/
       if ( np > 8 )
         {
         refadr = pv[9].par_va.lit.adr_va;
         strtyp(pv[9].par_ty,&typtbl);
         strarr(typtbl.arr_ty,&reftyp);
         dekl_dim = reftyp.up_arr - reftyp.low_arr + 1;
         if ( dekl_dim < nint ) return(erpush("IN5452",""));
         else for ( i=0; i<nint; ++i )
                evwrve(rlitpt[*(snumpt+i)].lit.ref_va,(short)i,(short)9,pv);
         }
       }
     }
/*
***Slut.
*/
   v3free(rlitpt,"evailm");
   if ( nint > 0 ) v3free(snumpt,"evailm");

   return(0);
}

/********************************************************/
/*!******************************************************/

        short evaios(
        PMPARVA  pv[],
        short    np)

/*      Evaluerar proceduren ALL_INV_ON_SUR.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          np   => Antal parametrar.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 13/9/95 J. Kjellander
 *
 ******************************************************!*/

{
   short   status;
   gmint   nint;
   DBVector   uv[25],xyz[25];
   PMLITVA litval;

/*
***Exekvera.
*/
   status = EXaios(&pv[1].par_va.lit.ref_va[0],
                   (DBVector *)&pv[2].par_va.lit.vec_va,
                    pv[3].par_va.lit.int_va,
                   &nint,uv,xyz);

   if ( status < 0 ) return(status);
/*
***Skriv in resultat i motsvarande MBS-variabler.
*/
   litval.lit.int_va = nint;
   inwvar(pv[4].par_ty,pv[4].par_va.lit.adr_va,0,NULL,&litval);

   if ( np > 4 )
     {
     if ( (status=evwvec(uv,(short)nint,(short)5,pv)) < 0 ) return(status);
     if ( np > 5 )
       {
       if ( (status=evwvec(xyz,(short)nint,(short)6,pv)) < 0 ) return(status);
       }
     }

   return(0);
}

/********************************************************/
/*!******************************************************/

        short evscur(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluates the geometry procedure SUR_CURVES.
 *
 *      In: ident    => Entity ID.
 *          pv       => Pointer to array with parameter values
 *          npblockp => Pointer to name parameter block.
 *
 *      Out: Nothing.
 *
 *      Errors:   IN5202 = Number of curves < 2.
 *                IN5212 = Number of curves > declared size.
 *                IN5222 = Unable to allocate memory.
 *
 *      FV: Returns status from called functions.
 *
 *      (C)microform ab 2000-11-01 Gunnar Liden
 *
 *      2000-12-01 Bugfix (ingref()), J.Kjellander
 *
 *
 ******************************************************!*/

{
   int      ncur;           /* Number of input curves (V lines) */
   int      nulin;          /* Number of requested U lines      */
   DBint    curadr;         /* Adress to input curves           */
   STTYTBL  typtbl;         /* Type of variable                 */
   STARRTY  curtyp;         /* Type of array for input curves   */
   int      dekl_dim;       /* Declared dimension for the array */
   int      refsiz;         /* Size of ????                     */
   PMLITVA  val;            /* Value from ??                    */
   PMREFVA *lpek;           /* Pointer to array with references */
   int      i;              /* Loop index                       */
   short    status;         /* Status from execution function   */
   char     errbuf[81];     /* String variable for errors       */

/*
***Requested number of U curves in the output surface
*/
   nulin = pv[1].par_va.lit.int_va;
/*
***Check that the number of U lines reasonable
*/
/* Not yet impl.    if ( nulin < 2 ) return(erpush("IN5202","")); */

/*
***How many input V curves.
*/
   ncur = pv[2].par_va.lit.int_va;
/*
***Check that number of input curves is reasonable
*/
   if ( ncur < 2 ) return(erpush("IN5202",""));
/*
***Check that the declared dimension of the MBS-array is sufficient for ncur
*/
/* Address to the variable                                */
   curadr = pv[3].par_va.lit.adr_va;
/* Get type of variable, which is defined in a structure  */
   strtyp(pv[3].par_ty,&typtbl);
/* Get type of array                                      */
   strarr(typtbl.arr_ty,&curtyp);
/* Get the declared size of the array                     */
   dekl_dim = curtyp.up_arr - curtyp.low_arr + 1;
/* Return with error if array not has sufficient size     */
   if ( dekl_dim < ncur ) return(erpush("IN5212",""));
/*
***Get size of ???
*/
   strtyp(curtyp.base_arr,&typtbl);
   refsiz = typtbl.size_ty;
/*
***Allocate memory.
*/
   if ( (lpek=(PMREFVA *)v3mall(ncur*sizeof(PMREFVA),"evscur")) == NULL )
     {
     sprintf(errbuf,"%d",ncur);
     return(erpush("IN5222",errbuf));
     }
/*
***Copy from RTS to the allocated area.
*/
   for ( i=0; i<ncur; ++i )
     {
     ingref(curadr+i*refsiz,&val);
     (lpek+i)->seq_val  = val.lit.ref_va[0].seq_val;
     (lpek+i)->ord_val  = val.lit.ref_va[0].ord_val;
     (lpek+i)->p_nextre = val.lit.ref_va[0].p_nextre;
     }
/*
***Call the execution function.
*/
   status = EXscur(ident,            /* Entity ID                    */
            (DBint)nulin,            /* Requested number of U lines  */
            (DBint)ncur,             /* Number of input curves       */
            lpek,                    /* Curve references             */
            pv[4].par_va.lit.str_va, /* Twist method                 */
            npblockp);               /* Name parameter block pointer */
/*
***Free memory
*/

   v3free(lpek,"evscur");


   return(status);
}
/********************************************************/
