/**********************************************************************
*
*    inac14.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    short evmolo()    evaluate procedure mode_local
*    short evmogl()    evaluate procedure mode_global
*    short evmoba()    evaluate procedure mode_basic 
*    short evset()     evaluate procedure SET 
*    short evsetb()    evaluate procedure SET_BASIC
*    short evos()      evaluate procedure OS 
*    short evexit()    evaluate procedure EXIT
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"

extern V2NAPA  defnap;       /* Gloabal area for named parameter values */

/********************************************************/
/*!******************************************************/

        short evmolo(
        PMPARVA  pv[])

/*      Evaluate procedure MODE_LOCAL.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: 
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   short status;
/*
***execute MODE_LOCAL 
*/
   if ( ( status = EXmolo(  &pv[1].par_va.lit.ref_va[0] ) ) < -1 )
      return( status );

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evmogl()

/*      Evaluate procedure MODE_GLOBAL.
 *
 *      In:   
 *
 *      Out: 
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXmogl());
  }

/********************************************************/
/*!******************************************************/

        short evmoba()

/*      Evaluate procedure MODE_BASIC.
 *
 *      In: 
 *
 *      Out: 
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
    return(EXmoba());
  } 

/********************************************************/
/*!******************************************************/

        short evset(
        pm_ptr exlist)


/*      Evaluate procedure SET.
 *
 *      In:   exlist  =>   PM-pointer to actual parameter(s)
 *
 *      Out: 
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   V2NAPA npblock;           /* named parameter block */
   short status;
/*
***evaluate named parameters
*/
   if ( ( status = inevnl( exlist, &npblock ) ) < -1 )
      return( status );
/*
***copy new values to "defnap" 
*/
   V3MOME((char *)&npblock,(char *)&defnap,sizeof(V2NAPA));

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evsetb(
        pm_ptr exlist)    /* in - PM-pointer to actual parameter(s) */


/*      Evaluate procedure SET_BASIC.
 *
 *      In:   exlist  => PM-pointer to actual parameter(s) 
 *
 *      Out: 
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1991-06-06 J. Kjellander
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   short status;
/* 
***evaluate named parameters and update defnap + stacked blocks 
*/
   if ( ( status = insetb( exlist ) ) < -1 )
      return( status );

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evos(
        PMPARVA  pv[],
        short    npar)

/*      Evaluate geometric procedure OS.
 *
 *      In:   pv[]  =>    Parameter value array
 *            npar  =>    Number of parameters
 *
 *      Out: 
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

 {
  short mode;

  if ( npar == 1 ) mode = 0;
  else mode = (short)pv[2].par_va.lit.int_va;

  return(EXos(pv[1].par_va.lit.str_va,mode));
 }  

/********************************************************/
/*!******************************************************/

        short evexit(
        PMPARVA  pv[],
        short    npar)

/*      Evaluate procedure EXIT.
 *
 *      In:   pv[]  =>    Parameter value array
 *            npar  =>    Number of parameters
 *
 *      Out: 
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***Returvrde = 3 => Slut med felmeddelande. Partanrop lagras ej.
***             4 => Slut utan felmeddelande. Partanrop lagras.
*** 1 anvnds fr GOTO och 2 fr IF.
*/
   if ( npar == 1 )
     {
     if ( strlen(pv[1].par_va.lit.str_va) > 0 )
       {
       erpush("IN2292",pv[1].par_va.lit.str_va);
       errmes();
       }
     return(3);
     }
   else return(4);
  }

/********************************************************/
