/**********************************************************************
*
*    ingeop.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    inevgp();    Interpreterar geometri-procedur
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../include/mbsrout.h"
#include <math.h>

/********************************************************/
/*!******************************************************/

        short inevgp(
        V2REFVA *identp,
        pm_ptr routla,
        pm_ptr arglist,
        V2NAPA *npblockp)

/*      Central rutin fr interpretering av geometri-
 *      procedurer.
 *
 *      In: identp   => Storhetens ID.
 *          routla   => Pekare till rutin.
 *          arglist  => Pekare till aktuell parameterlista.
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 23/3/86 J. Kjellander
 *
 ******************************************************!*/

{
   STPROC rout;                     /* Access structure for MBS routines */
   PMPARVA *pv;                     /* pv, array for parameter values */
   short parcount;                  /* number of actual parameters */
   short status;

/*
***Ls rutin-info frn symboltabell.
*/
   if ( ( status = strrou( routla, &rout ) ) < -1 )
       return(status);
/*
***Interpretera rutinens parameterlista.
*/
   if ( ( status = inapar( arglist, routla, &rout, &pv, &parcount ) ) != 0 )
      {
      inrpar();
      return(status);
      }
/*
***Exekvera.
*/
   switch ( rout.kind_pr )
     {
     case VPOI:
     case VPOIP:
     status = evpofr( identp, pv, rout.kind_pr, npblockp );
     break;

     case VLINF:
     case VLINP:
     status = evlifr( identp, pv, rout.kind_pr, npblockp );
     break;

     case VLINO:
     status = evliof( identp, pv, npblockp );
     break;

     case VLINA:
     status = evlian( identp, pv, npblockp );
     break;

     case VLINT1:
     status = evlit1( identp, pv, npblockp );
     break;

     case VLINT2:
     status = evlit2( identp, pv, npblockp );
     break;

     case VLINPER:
     status = evlipe( identp, pv, npblockp );
     break;

     case VARCF:
     status = evarfr( identp, pv, npblockp );
     break;

     case VARCO:
     status = evarof( identp, pv, npblockp );
     break;

     case VARC2P:
     status = evar2p( identp, pv, npblockp );
     break;

     case VARC3P:
     status = evar3p( identp, pv, npblockp );
     break;

     case VARCFI:
     status = evarfl( identp, pv, npblockp );
     break;

     case VCURF:
     case VCURP:
     status = evcufr(identp,pv,rout.kind_pr,npblockp,parcount);
     break;

     case VCURSA:
     status = evcusa(identp,pv,npblockp,parcount);
     break;

     case VCURC:
     status = evcurc(identp,pv,npblockp,parcount);
     break;

     case VCURCA:
     status = evcuca(identp,pv,npblockp);
     break;

     case VCOMP:
     status = evcomp(identp,pv,npblockp,parcount);
     break;

     case VCURS:
     status = evcurs(identp,pv,npblockp,parcount);
     break;

     case VCURO:
     status = evcuro(identp,pv,npblockp);
     break;

     case VCURT:
     status = evcurt(identp,pv,npblockp);
     break;

     case VCURU:
     status = evcuru(identp,pv,npblockp,parcount);
     break;

     case VCURSIL:
     status = evcsil(identp,pv,npblockp);
     break;

     case VCURINT:
     status = evcint(identp,pv,npblockp);
     break;

     case VCURISO:
     status = evciso(identp,pv,npblockp);
     break;

     case VCURFA:
     status = evcfan(identp,pv,npblockp,parcount);
     break;

     case VCURGD:
     status = evcged(identp,pv,npblockp);
     break;

     case VCURAP:
     status = evcapp(identp,pv,npblockp,parcount);
     break;

     case VCURGL:
     status = evcugl(identp,pv,npblockp);
     break;

     case VSUREX:
     status = evsexd(identp,pv,npblockp);
     break;

     case VSURCON:
     status = evscon(identp,pv,npblockp,parcount);
     break;

     case VSUROF:
     status = evsoff(identp,pv,npblockp);
     break;

     case VSURSA:
     status = evssar(identp,pv,npblockp,parcount);
     break;

     case VSURCA:
     status = evscar(identp,pv,npblockp);
     break;

     case VSURNA:
     status = evsnar(identp,pv,npblockp,parcount);
     break;

     case VSURAP:
     status = evsapp(identp,pv,npblockp,parcount);
     break;

     case VSURCO:
     status = evscom(identp,pv,npblockp,parcount);
     break;

     case VSURT:
     status = evstrm(identp,pv,npblockp,parcount);
     break;

     case VSURROT:
     status = evsrot(identp,pv,npblockp);
     break;

     case VSURCYL:
     status = evscyl(identp,pv,npblockp);
     break;

     case VSURSWP:
     status = evsswp(identp,pv,npblockp);
     break;

     case VSURRUL:
     status = evsrul(identp,pv,npblockp);
     break;

     case VSURUSD:
     status = evsusr(identp,pv,npblockp);
     break;

     case VSURCURV:
     status = evscur(identp,pv,npblockp);
     break;

     case VCSYS3P:
     status = evcs3p(identp,pv,npblockp,parcount);
     break;

     case VCSYS1P:
     status = evcs1p( identp, pv, npblockp, parcount);
     break;

     case VBPLAN:
     status = evbpln( identp, pv, npblockp );
     break;

     case VTEXT:
     status = evtext( identp, pv, npblockp );
     break;

     case VGROUP:
     status = evgrp( identp, pv, npblockp, parcount );
     break;

     case VLDIM:
     status = evldim( identp, pv, npblockp );
     break;

     case VCDIM:
     status = evcdim( identp, pv, npblockp );
     break;

     case VRDIM:
     status = evrdim( identp, pv, npblockp );
     break;

     case VADIM:
     status = evadim( identp, pv, npblockp );
     break;

     case VXHT:
     status = evxht( identp, pv, npblockp, parcount );
     break;

     case VSYMB:
     status = evsymb( identp, pv, npblockp );
     break;

     case VTFMOV:
     status = evtrmv( identp, pv, npblockp, parcount );
     break;

     case VTFROTP:
     status = evtrrp( identp, pv, npblockp, parcount );
     break;

     case VTFROTL:
     status = evtrrl( identp, pv, npblockp, parcount );
     break;

     case VTFMIRR:
     status = evtrmi( identp, pv, npblockp, parcount );
     break;

     case VTFCOMP:
     status = evtrco( identp, pv, npblockp );
     break;

     case VTFU:
     status = evtrus( identp, pv, npblockp );
     break;

     case VTCOPY:
     status = evtcop( identp, pv, npblockp );
     break;

     default:
     inrpar();
     return( erpush( "IN0104" , "" ) );
     break;
     }
/*
***Frigr parameter-pool utrymme.
*/
   inrpar();
/*
***Felkontroll.
*/
   if ( status == 0 ) return(0);
   else if ( status == -1 ) return(0);
   else return(status);
}

/************************************************************************/
